/*****************************************************************************/
/**
 *  @file   main.cpp
 *  @brief  Test program for kvs::String class.
 *  @author Naohisa Sakamoto
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: main.cpp 602 2010-08-19 02:43:34Z naohisa.sakamoto $
 */
/*****************************************************************************/
#include <iostream>
#include <kvs/String>

/*===========================================================================*/
/**
 *  @brief  Main function.
 */
/*===========================================================================*/
int main( void )
{
    const char* data = "Hello";

    kvs::String s1( data );
    std::cout << "kvs::String: " << s1 << std::endl;
    std::cout << "\tsize: " << s1.size() << std::endl;
    std::cout << "\tchar(0): " << s1.at(0) << std::endl;
    std::cout << "\tchar(1): " << s1.at(1) << std::endl;
    std::cout << "\tchar(2): " << s1.at(2) << std::endl;
    std::cout << "\tchar(3): " << s1.at(3) << std::endl;
    std::cout << "\tchar(4): " << s1.at(4) << std::endl;

    std::string s2( s1.data() );
    std::cout << "std::string:  " << s2 << std::endl;
    std::cout << "\tsize: " << s2.size() << std::endl;
    std::cout << "\tchar(0): " << s2.at(0) << std::endl;
    std::cout << "\tchar(1): " << s2.at(1) << std::endl;
    std::cout << "\tchar(2): " << s2.at(2) << std::endl;
    std::cout << "\tchar(3): " << s2.at(3) << std::endl;
    std::cout << "\tchar(4): " << s2.at(4) << std::endl;

    s1.upper();
    std::cout << "kvs::String::upper: " << s1 << std::endl;

    s1.lower();
    std::cout << "kvs::String::lower: " << s1 << std::endl;

    s1.upper(0);
    std::cout << "kvs::String::upper(0): " << s1 << std::endl;

    s1.upper(2);
    std::cout << "kvs::String::upper(2): " << s1 << std::endl;

    s1.lower(2);
    std::cout << "kvs::String::lower(2): " << s1 << std::endl;

    s1 += " KVS";
    std::cout << "kvs::String:: += ' KVS': " << s1 << std::endl;

    kvs::String s3;

    int    v1 = 12;
    double v2 = 12.3;
    s3.format( "int %d, double %lf", v1, v2 );
    std::cout << "kvs::String: " << s3 << std::endl;

    kvs::String s4( s3 );
    std::cout << "s1 = " << s1 << std::endl;
    std::cout << "s3 = " << s3 << std::endl;
    std::cout << "s4 = " << s4 << std::endl;
    std::cout << "s1 == s3 ? " << ( s1 == s3 ? "true" : "false" ) << std::endl;
    std::cout << "s1 != s3 ? " << ( s1 != s3 ? "true" : "false" ) << std::endl;
    std::cout << "s1 >  s3 ? " << ( s1 >  s3 ? "true" : "false" ) << std::endl;
    std::cout << "s1 <  s3 ? " << ( s1 <  s3 ? "true" : "false" ) << std::endl;
    std::cout << "s3 == s4 ? " << ( s3 == s4 ? "true" : "false" ) << std::endl;
    std::cout << "s3 != s4 ? " << ( s3 != s4 ? "true" : "false" ) << std::endl;

    kvs::String s5( 1234 );
    std::cout << "kvs::String: " << s5 << std::endl;

    kvs::String s6( 1234.5 );
    std::cout << "kvs::String: " << s6 << std::endl;

    s6.setNumber( v2 );
    std::cout << "kvs::String: " << s6 << std::endl;

    int v3 = kvs::String::toNumber<int>("12345");
    std::cout << "kvs::String::toNumber: " << v3 << std::endl;

    kvs::String s("XXX XXX YYY.");
    s.replace( "XXX", "abc" );
    s.replace( "YYY", "defghijklmn" );
    std::cout << s << std::endl;

    return 0;
}
