/*****************************************************************************/
/**
 *  @file   TFE_Color_3.h
 *  @author Naohisa Sakamoto
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: TFE_Color_3.h 634 2010-10-13 07:04:05Z naohisa.sakamoto $
 */
/*****************************************************************************/
#ifndef KVS__GLUT__TFE_COLOR_3_H_INCLUDE
#define KVS__GLUT__TFE_COLOR_3_H_INCLUDE

#include <string>
#include <kvs/ClassName>
#include <kvs/OpenGL>
#include <kvs/RGBColor>
#include <kvs/RGBAColor>
#include <kvs/Vector2>
#include <kvs/TransferFunction>
#include <kvs/Texture1D>
#include <kvs/Texture2D>
#include <kvs/MouseReleaseEventListener>
#include <kvs/glut/Screen>
#include <kvs/glut/PushButton>
#include <kvs/glut/Text>
#include <kvs/glut/ColorMapPalette>
#include <kvs/glut/Histogram>

#ifdef __CYGWIN__    //For Cygwin on Windows
#  include <gl/glui.h> //maybe
#else               //For Linux
#  include "glui.h"
#endif

namespace kvs
{

namespace visclient
{

  class TFE_main;

/*===========================================================================*/
/**
 *  @brief  Transfer function editor class.
 */
/*===========================================================================*/
class TFE_Color_3 : public kvs::glut::Screen
{
    kvsClassName( kvs::visclient::TFE_Color_3 );

public:

    typedef kvs::glut::Screen SuperClass;

protected:

    bool m_visible;
    size_t m_resolution;

    kvs::ScreenBase*              m_screen;              ///< pointer to screen
    kvs::glut::ColorMapPalette*   m_color_map_palette;   ///< color map palette
    kvs::glut::Histogram*         m_histogram;           ///< histogram
    kvs::TransferFunction m_initial_transfer_function;   ///< initial transfer function
    kvs::Real32           m_min_value; ///< min value
    kvs::Real32           m_max_value; ///< max value

    kvs::visclient::TFE_main* m_parent;

    GLUI* m_glui_text_win;
    GLUI* m_glui_button_win;
    GLUI_EditText* m_glui_edit_c[10];
    GLUI_EditText* m_glui_edit_r[10];
    GLUI_EditText* m_glui_edit_g[10];
    GLUI_EditText* m_glui_edit_b[10];

public:

    TFE_Color_3( kvs::visclient::TFE_main* parent = 0 );

    virtual ~TFE_Color_3( void );

public:

    int show( void );

    virtual void hide( void );

public:

    void setResolution( size_t resolution );

public:

    kvs::ScreenBase* screen( void );

    const kvs::glut::ColorMapPalette* colorMapPalette( void ) const;

    const kvs::ColorMap colorMap( void ) const;

    const kvs::TransferFunction transferFunction( void ) const;

public:

    void setTransferFunction( const kvs::TransferFunction& transfer_function );

    void setVolumeObject( const kvs::VolumeObjectBase* object );

public:

    //void setControlPointValue( size_t n,  float cp, kvs::UInt8 r, kvs::UInt8 g, kvs::UInt8 b );
    void setControlPointValue( size_t n,  float cp, float r, float g, float b );

public:

    virtual void update( void );

    virtual void save( void );

    virtual void cancel( void );
};

} // end of namespace visclient

} // end of namespace kvs

#endif // KVS__GLUT__TFE_COLOR_3_H_INCLUDE
