#include "kvs/glut/OrientationAxis"
#include "TimerEvent.h"
#include "Command.h"
#include "sighandler.h"
#include "KeyFrameAnimation.h"
#include "ParticleMerger.h"
#include "ParticleServer.h"
#include "PVRenderer.h"
#include "Controller.h"
#include "TFE_main.h"

void update_sending();
void update_recieving();
void update_gmemory();
static kvs::Xform interpolateXform( int interp_step, int num_frame, kvs::Xform start, kvs::Xform end );
static float SIGN(float x);
static float NORM(float a, float b, float c, float d);
static kvs::Quaternion<float> RtoQ( kvs::Matrix33f R );
extern kvs::glut::OrientationAxis* g_orientation_axis;
extern int ValueInterpolation;
extern      void  ScreenShotKeyFrame( kvs::ScreenBase* screen, int tstep );

#if 1 // FEAST modify 20160128
extern int g_curStep;
#endif // FEAST modify 20160128 end


using namespace kvs::visclient;

TimerEvent::TimerEvent(Command* command, ComThread* comthread):
    command(command),
    m_comthread(comthread)
{
  m_front_object = NULL;
  isKeyFrameAnimation = 0;
  isReady = 0;
  m_interpolation_counter = 0;
  m_ninterpolation = 10;
  m_xform_index = 0;
  KeyFrameAnimation_init();
  m_xforms = KeyFrameAnimation_getXforms();
  m_timeSteps = KeyFrameAnimation_getTimeSteps();
}

#ifdef CPUMODE
void TimerEvent::setObject(const int &id)
{
    m_object_id = id;
}

void TimerEvent::setRenderer(kvs::ParticleVolumeRenderer* renderer)
{
	m_renderer = renderer;
}
TimerEvent::~TimerEvent()
{
	if( m_back_object  != NULL ) delete m_back_object;
	if( m_front_object != NULL ) delete m_front_object;
}
#endif

void TimerEvent::update(kvs::TimeEvent* event)
{
	const ComThread::ComStatus comstatus = m_comthread->getStatus();
	if (comstatus == ComThread::Idle) {
	    command->preupdate();
	    m_comthread->start();
	} else if (comstatus == ComThread::Exit) {
	    m_comthread->wait();
	    m_comthread->quit();
	    command->postupdate();
#ifdef CPUMODE
	    m_front_object = new PointObject();

	    if (command->param.transferType == PBVRParam::Abstract) {
	        *m_front_object = *(command->abstractParticles[command->param.timeStep]);
	        m_renderer->setSubpixelLevel(command->param.abstractSubpixelLevel);
	    }
	    else if (command->param.transferType == PBVRParam::Detailed) {
	        *m_front_object = *(command->detailedParticles[command->param.timeStep]);
	        m_renderer->setSubpixelLevel(command->param.detailedSubpixelLevel);
	    }
	    else
	    {
		assert(false);
	    }
            if( command->param.shadingType_f )
                m_renderer->enableShading();
	    m_renderer->attachPointObject(m_front_object);
	    m_back_object = m_front_object;
    	    screen()->objectManager()->change(m_object_id, m_front_object);
          g_orientation_axis->setObject(m_front_object);
#endif
          if(command->param.timeStep == command->param.timeStepKeyFrame){
            isReady = 1;
          }

#if 1 // FEAST modify 20160128
	  if( command->isUnderCommunication ) {
	    if(command-> param.transferType == PBVRParam::Abstract ) {
	      command->progress->setValue( static_cast<float>(command->param.timeStep) );
	      command->timeSlider->setValue( static_cast<float>(command->param.timeStep) );
	    }
	    else if( command->param.transferType == PBVRParam::Detailed )
	      {
		if( command->param.timeStep > command->param.maxTimeStep ) {
		  command->param.timeStep = command->param.minTimeStep;
		}
		float ratio = float(command->param.timeStep - command->param.minTimeStep) / float(command->param.maxTimeStep - command->param.minTimeStep) * 100;
		command->progress->setValue( static_cast<float>(ratio) );
		command->timeSlider->setValue( static_cast<float>(command->param.timeStep) );
	      }
	  }
#endif // FEAST modify 20160128 end

#if 1 // FEAST modify 20160128
	  if( command->isUnderAnimation ) {
	    if( command->param.transferType == PBVRParam::Detailed ) {
#ifndef CPUMODE
	      if( command->param.repeatLevel == command->param.detailedRepeatLevel ) {
#endif
		g_curStep = command->param.timeStep;
		command->param.timeStep++;
#ifndef CPUMODE
	      }
#endif
	    }
	    else if( command->param.transferType == PBVRParam::Abstract ) {
#ifndef CPUMODE
	      if( command->param.repeatLevel == command->param.abstractRepeatLevel ) {
#endif
		g_curStep = command->param.timeStep;
		command->param.timeStep++;
#ifndef CPUMODE
	      }
#endif
	    }
	  }
	  if( command->param.timeStep > command->maxTimeSlider->value()) {//99) {
	    if(command->minTimeSlider->value() >= command->param.minTimeStep){
	      command->param.timeStep = command->minTimeSlider->value();
	    } else {
	      command->param.timeStep = command->param.minTimeStep;
	    }
	  }
#endif // FEAST modify 20160128 end

	} else if (comstatus == ComThread::Running) {
	    const ParticleServer::Status srvstat = command->server->getStatus();
	    if (srvstat == ParticleServer::Sending)   update_sending();
	    if (srvstat == ParticleServer::Recieving) update_recieving();
	    update_gmemory();
	}

      if(isKeyFrameAnimation){
        m_ninterpolation = ValueInterpolation;
        if(m_interpolation_counter >= m_ninterpolation){
          m_interpolation_counter = 0;
          m_xform_index++;
        }
        if( m_xform_index < (int)m_xforms->size()-1 ) {
          int i = m_xform_index;
          int t = m_interpolation_counter;
          float delta = (float)(m_timeSteps->at(i+1) - m_timeSteps->at(i)) / (float)m_ninterpolation;
          float f = delta * (float)t;
          command->param.timeStepKeyFrame = (int)((float)(m_timeSteps->at(i)) + f);
          
          if(command->param.CSmode == 1){
            if(command->param.timeStep != command->param.timeStepKeyFrame){
              isReady = 0;
            }
            if(m_front_object && isReady){
              if( m_interpolation_counter < m_ninterpolation ) {
                kvs::Xform Xform_new = interpolateXform( t, m_ninterpolation, m_xforms->at(i), m_xforms->at(i+1) );
                m_front_object->setXform( Xform_new );
                command->stepKeyFrame++;
                t++;
                m_interpolation_counter = t;
              }
            }
          } else {
            if(command->param.timeStep != command->param.timeStepKeyFrame){
              isReady = 0;
            }
            if(m_front_object && isReady){
              if( m_interpolation_counter < m_ninterpolation ) {
                kvs::Xform Xform_new = interpolateXform( t, m_ninterpolation, m_xforms->at(i), m_xforms->at(i+1) );
                m_front_object->setXform( Xform_new );
                command->stepKeyFrame++;
                t++;
                m_interpolation_counter = t;
              }
            }
          }
        } else {
          m_xform_index = 0;
          m_interpolation_counter = 0;
          isKeyFrameAnimation = 0;
          command->isKeyFrameAnimation = false;
          command->stepKeyFrame = 0;
          command->preKeyFrame = 0;
        }
      }
      
#ifdef CPUMODE
        screen()->redraw();
#endif
        command->reDraw();
        
      if(isKeyFrameAnimation && command->preKeyFrame != command->stepKeyFrame){
        ScreenShotKeyFrame( command->screen, command->stepKeyFrame );
        command->preKeyFrame = command->stepKeyFrame;
      }

	if (issig()) {
	    if (command->param.CSmode == 1) {
	        command->close_server();
	    }
	    exit(1);
	}

}

static kvs::Xform interpolateXform( int interp_step, int num_frame, kvs::Xform start, kvs::Xform end )
{
  // range of the interpolation parametar t = [0,1].
  float delta = 1.0 / (float)num_frame;
  float t = delta * (float)interp_step;

  kvs::Matrix33f rotation_0( start.rotation() );
  kvs::Vector3f translation_0( start.translation() );
  kvs::Vector3f scaling_0( start.scaling() );

  kvs::Matrix33f rotation_1( end.rotation() );
  kvs::Vector3f translation_1( end.translation() );
  kvs::Vector3f scaling_1( end.scaling() );

  kvs::Quaternion<float> q_0 = RtoQ( rotation_0 );
  kvs::Quaternion<float> q_1 = RtoQ( rotation_1 );
  kvs::Quaternion<float> q =
    kvs::Quaternion<float>::sphericalLinearInterpolation( q_0, q_1, t, true, true );
  kvs::Matrix33f rotation = q.toMatrix();

  kvs::Vector3f translation = translation_1 * t + translation_0 * (1-t);

  kvs::Vector3f scaling = scaling_1 * t + scaling_0 * (1-t);

  kvs::Xform xform( translation, scaling, rotation );
  return xform;
}

static float SIGN(float x)
{
  return (x >= 0.0f) ? +1.0f : -1.0f;
}

static float NORM(float a, float b, float c, float d)
{
  return sqrt(a * a + b * b + c * c + d * d);
}
//High precision conversion of Rotation Matrix to Quaternion
static kvs::Quaternion<float> RtoQ( kvs::Matrix33f R )
{
  float r11 = R[0][0]; float r12 = R[0][1]; float r13 = R[0][2];
  float r21 = R[1][0]; float r22 = R[1][1]; float r23 = R[1][2];
  float r31 = R[2][0]; float r32 = R[2][1]; float r33 = R[2][2];

  float q0 = ( r11 + r22 + r33 + 1.0f) / 4.0f;
  float q1 = ( r11 - r22 - r33 + 1.0f) / 4.0f;
  float q2 = (-r11 + r22 - r33 + 1.0f) / 4.0f;
  float q3 = (-r11 - r22 + r33 + 1.0f) / 4.0f;
  if(q0 < 0.0f) q0 = 0.0f;
  if(q1 < 0.0f) q1 = 0.0f;
  if(q2 < 0.0f) q2 = 0.0f;
  if(q3 < 0.0f) q3 = 0.0f;
  q0 = sqrt(q0);
  q1 = sqrt(q1);
  q2 = sqrt(q2);
  q3 = sqrt(q3);
  if(q0 >= q1 && q0 >= q2 && q0 >= q3) {
    q0 *= +1.0f;
    q1 *= SIGN(r32 - r23);
    q2 *= SIGN(r13 - r31);
    q3 *= SIGN(r21 - r12);
  } else if(q1 >= q0 && q1 >= q2 && q1 >= q3) {
    q0 *= SIGN(r32 - r23);
    q1 *= +1.0f;
    q2 *= SIGN(r21 + r12);
    q3 *= SIGN(r13 + r31);
  } else if(q2 >= q0 && q2 >= q1 && q2 >= q3) {
    q0 *= SIGN(r13 - r31);
    q1 *= SIGN(r21 + r12);
    q2 *= +1.0f;
    q3 *= SIGN(r32 + r23);
  } else if(q3 >= q0 && q3 >= q1 && q3 >= q2) {
    q0 *= SIGN(r21 - r12);
    q1 *= SIGN(r31 + r13);
    q2 *= SIGN(r32 + r23);
    q3 *= +1.0f;
  } else {
    printf("coding error\n");
  }
  float r = NORM(q0, q1, q2, q3);
  q0 /= r;
  q1 /= r;
  q2 /= r;
  q3 /= r;

  return kvs::Quaternion<float>( q1, q2, q3, q0 );
}
