#ifndef DISTRIBUTED_REJECTION_SAMPLING_H
#define DISTRIBUTED_REJECTION_SAMPLING_H

#include <kvs/UnstructuredVolumeObject>

namespace kvs {
namespace test {

class DistributedRejectionSampling {

private:
	static void generate_particles_core_gpu(
		const kvs::UnstructuredVolumeObject *volume,
		const float *density_map,
		const unsigned char *color_map,
		size_t resolution, float min_value, float max_value,
		kvs::ValueArray<kvs::Real32> *vertex_coords,
		kvs::ValueArray<kvs::UInt8>  *vertex_colors,
		kvs::ValueArray<kvs::Real32> *vertex_normals);

public:
	static void generate_particles_master_gpu(
		const kvs::UnstructuredVolumeObject *volume,
		const float *density_map,
		const unsigned char *color_map,
		size_t resolution, float min_value, float max_value,
		kvs::ValueArray<kvs::Real32> &vertex_coords,
		kvs::ValueArray<kvs::UInt8>  &vertex_colors,
		kvs::ValueArray<kvs::Real32> &vertex_normals);

	static void generate_particles_slave_gpu();

	/*** debug ***/
	static void generate_particles_slave_ts_gpu(
	        const kvs::UnstructuredVolumeObject *volume);
	/*************/

private:
	static void generate_particles_core_cpu(
		const kvs::UnstructuredVolumeObject *volume,
		const float *density_map,
		const unsigned char *color_map,
		size_t resolution, float min_value, float max_value,
		kvs::ValueArray<kvs::Real32> *vertex_coords,
		kvs::ValueArray<kvs::UInt8>  *vertex_colors,
		kvs::ValueArray<kvs::Real32> *vertex_normals);

	static float calculate_density(
		const float scalar, const float min_value, const float max_value,
		const size_t resolution, const float *density_map);

	static size_t calculate_number_of_particles(
		const float density, const float volume_of_cell);

	static float calculate_maximum_density(
		const float scalar0, const float scalar1,
		const float min_value, const float max_value, const size_t resolution,
		const float *density_map);

public:
	static void generate_particles_master_cpu(
		const kvs::UnstructuredVolumeObject *volume,
		const float *density_map,
		const unsigned char *color_map,
		size_t resolution, float min_value, float max_value,
		kvs::ValueArray<kvs::Real32> &vertex_coords,
		kvs::ValueArray<kvs::UInt8>  &vertex_colors,
		kvs::ValueArray<kvs::Real32> &vertex_normals);

	static void generate_particles_slave_cpu();

};

} // end of namespace test
} // end of namespace kvs

#endif

