#ifndef FRONT_STR_FILE_READER_H
#define FRONT_STR_FILE_READER_H

#include <kvs/UnstructuredVolumeObject>
#include <kvs/ValueArray>
#include <kvs/Type>

namespace kvs
{

class FrontSTRFileReader:
    public kvs::UnstructuredVolumeObject
{

public:

    enum ConnectionReadMode
    {
        Skip,		///Read connection data every several steps
        Block,      ///Read assigned connection data from some part 
    };

private:

    int m_process_id;
    int m_nprocesses;
    int m_vpower;

    ConnectionReadMode m_connection_read_mode;
    CellType           m_cell_type;

    int m_max_length;

    float* m_assigned_coordsdata;
    float* m_assigned_nodesdata;

    int m_assigned_nconnections;
    int m_assigned_nvalues;
    int m_assigned_ncoords;

    char m_connections_filename[256];
    char m_coords_filename[256];
    char m_nodes_filename[256];

    int m_each_nnodes;

    kvs::ValueArray<kvs::UInt32>   m_assigned_connections;
    kvs::ValueArray<kvs::Real32>   m_assigned_values;
    kvs::ValueArray<float>         m_assigned_coords;

public:

    FrontSTRFileReader( const char* dirname,
                        const int process_id, 
                        const int nprocesses,
                        const ConnectionReadMode connection_read_mode,
                        const CellType cell_type,
                        const int vpower = 1,
                        const int m_max_length = 256);

    ~FrontSTRFileReader( void );

    void read( void );

private:

    kvs::ValueArray<kvs::UInt32>   getConnections_SkipMode( void );
    kvs::ValueArray<kvs::UInt32>   getConnections_BlockMode( void );
    kvs::ValueArray<float>		   getCoords( void );
    kvs::ValueArray<kvs::Real32>   getValues( void );

public:

    int getAssignedNConnections( void )
    {
        return m_assigned_nconnections;
    }

    int getAssignedNValues( void )
    {
        return m_assigned_nvalues;
    }

    int getAssignedNCoords( void )
    {
        return m_assigned_ncoords;
    }

    CellType getCellType( void )
    {
        return m_cell_type;
    }

    int getEachNNodes( void )
    {
        return m_each_nnodes;
    }

    kvs::ValueArray<kvs::UInt32>& getAssignedConnections( void )
    {
        return( m_assigned_connections );
    }

    kvs::ValueArray<kvs::Real32>& getAssignedValues( void )
    {
        return( m_assigned_values );
    }

    kvs::ValueArray<float>& getAssignedCoords( void ){
        return( m_assigned_coords );
    }

};

} // end of namespace kvs

#endif // FRONT_STR_FILE_READER_H
