
#include "ComThread.h"

kvs::visclient::ComThread::ComThread(Command* command) :
    m_command(command),
    m_status(Idle)
{
}

kvs::visclient::ComThread::~ComThread()
{
}

bool kvs::visclient::ComThread::start()
{
    bool ret;
    make_param();
    ret = kvs::Thread::start();
    if (ret) setStatus(Running);

    return ret;
}

bool kvs::visclient::ComThread::wait()
{
    bool ret = kvs::Thread::wait();
    if (ret) {
        setStatus(Idle);
        emit_result();
    }
    return ret;
}

void kvs::visclient::ComThread::quit()
{
    kvs::Thread::quit();
    setStatus(Idle);
}

void kvs::visclient::ComThread::run()
{
    setStatus(Running);

    m_mutex_param.lock();
    m_mutex_result.lock();

    m_command->update(m_param,&m_result);

    m_mutex_result.unlock();
    m_mutex_param.unlock();

    setStatus(Exit);
}

void kvs::visclient::ComThread::make_param()
{
    m_mutex_param.lock();
    m_param = m_command->param;
    m_mutex_param.unlock();

    setStatus(Ready);
}

void kvs::visclient::ComThread::emit_result()
{
    m_mutex_result.lock();
    m_command->result = m_result;
    m_mutex_result.unlock();

    setStatus(Ready);
}

void kvs::visclient::ComThread::setStatus(const ComStatus status)
{
    m_mutex_status.lock();
    m_status = status;
    m_mutex_status.unlock();
}

kvs::visclient::ComThread::ComStatus kvs::visclient::ComThread::getStatus()
{
    ComStatus ret;
    m_mutex_status.lock();
    ret = m_status;
    m_mutex_status.unlock();
    return ret;
}

