#include <iostream>
#include <sstream>
#include "ParamExTransFunc.h"
#include "ExtendTransferFunction.h"

using kvs::visclient::ParamExTransFunc;

ParamExTransFunc::ParamExTransFunc() :
    transferFunction(NPRTF),
    range_initialized(NPRTF,false)
{
    ExTransFuncParaSet();
}

ParamExTransFunc::~ParamExTransFunc()
{
}

void ParamExTransFunc::ExTransFuncParaSet()
{
    ext_transferFunctionResolution = 256;
    for (size_t i = 0; i < transferFunction.size(); i++ ) {
        std::stringstream ss;
        ss << "t" << i+1;
        transferFunction[i] = kvs::visclient::NamedTransferFunction();
        transferFunction[i].Name       = ss.str();
        transferFunction[i].ColorVar   = "q1";
        transferFunction[i].OpacityVar = "q1";

        range_initialized[i] = false;
    }
    ext_transferFunctionSynthesis = "t1";
    ext_volumeSynthesis           = "";
    return;
}

void ParamExTransFunc::ExTransFuncParaDump()
{
    std::cout << "transferFunctionResolution " << ext_transferFunctionResolution << std::endl;
    for(size_t i = 0; i < transferFunction.size(); i++ ) {
        std::cout << "transferFunction[" << i << "].Name          " << transferFunction[i].Name << std::endl;
        std::cout << "transferFunction[" << i << "].ColorVar      " << transferFunction[i].ColorVar  << std::endl;
        std::cout << "transferFunction[" << i << "].ColorVarMin   " << transferFunction[i].ColorVarMin << std::endl;
        std::cout << "transferFunction[" << i << "].ColorVarMax   " << transferFunction[i].ColorVarMax << std::endl;
        std::cout << "transferFunction[" << i << "].OpacityVar    " << transferFunction[i].OpacityVar  << std::endl;
        std::cout << "transferFunction[" << i << "].OpacityVarMin " << transferFunction[i].OpacityVarMin << std::endl;
        std::cout << "transferFunction[" << i << "].OpacityVarMax " << transferFunction[i].OpacityVarMax << std::endl;
        std::cout << "transferFunction[" << i << "].EquationR     " << transferFunction[i].EquationR << std::endl;
        std::cout << "transferFunction[" << i << "].EquationG     " << transferFunction[i].EquationG << std::endl;
        std::cout << "transferFunction[" << i << "].EquationB     " << transferFunction[i].EquationB << std::endl;
        std::cout << "transferFunction[" << i << "].EquationA     " << transferFunction[i].EquationA << std::endl;
    }
    std::cout << "transferFunctionSynthesis " << ext_transferFunctionSynthesis << std::endl;
    std::cout << "volumeSynthesis " << ext_volumeSynthesis << std::endl;
}

void ParamExTransFunc::applyToClientMessage(jpv::ParticleTransferClientMessage* message) const
{
    message->transfunc.clear();
    message->voleqn.clear();

    for(size_t i=0; i<ParamExTransFunc::NPRTF; i++) {
        kvs::visclient::NamedTransferFunction etf;
        jpv::ParticleTransferClientMessage::VolumeEquation veq_c,veq_o;

        const NamedTransferFunction& tf = this->transferFunction[i];

        etf = tf;
        if ( etf.ColorVar   == "" ) { continue; }
        if ( etf.OpacityVar == "" ) { continue; }

        std::stringstream ss;
        ss << "_F" << i;

        etf.ColorVar   = ss.str() + "_VAR_C";
        etf.OpacityVar = ss.str() + "_VAR_O";
        veq_c.Name     = etf.ColorVar;
        veq_o.Name     = etf.OpacityVar;
        veq_c.Equation = this->transferFunction[i].ColorVar;
        veq_o.Equation = this->transferFunction[i].OpacityVar;

        message->transfunc.push_back(etf);
        message->voleqn.push_back(veq_c);
        message->voleqn.push_back(veq_o);
    }
    message->transferFunctionSynthesis = this->ext_transferFunctionSynthesis;
}

