#ifndef __KVS_VISCLIENT_TIMER_EVENT_H__
#define __KVS_VISCLIENT_TIMER_EVENT_H__

#include "kvs/TimerEventListener"
#include "kvs/PointObject"
#include "kvs/ParticleVolumeRenderer"
#include "Command.h"
#include "ComThread.h"

namespace kvs{
	namespace visclient{
		class Command;

class TimerEvent : public kvs::TimerEventListener
{
private:
	Command* command;
        ComThread* m_comthread;
#ifdef CPUMODE
	int m_object_id;
	kvs::PointObject* m_front_object;
	kvs::PointObject* m_back_object;
	kvs::ParticleVolumeRenderer* m_renderer;
#endif
      int m_interpolation_counter;
      int m_ninterpolation;
      int isKeyFrameAnimation;
      int isReady;
      int m_xform_index;
      std::vector<kvs::Xform>* m_xforms;
      std::vector<int> *m_timeSteps;

public:
        TimerEvent(Command* command, ComThread* comthread);
    void update( kvs::TimeEvent* event);
#ifdef CPUMODE
	~TimerEvent();
	void setObject(const int &id);
	void setRenderer(kvs::ParticleVolumeRenderer* renderer);
      kvs::PointObject* getPointObject()
         {
            return m_front_object;
         }
#endif
      void enableKeyFrameAnimation(){
         isKeyFrameAnimation = 1;
         command->isKeyFrameAnimation = true;
      }
      void disableKeyFrameAnimation(){
         m_xform_index = 0;
         m_interpolation_counter = 0;
         isKeyFrameAnimation = 0;
         command->isKeyFrameAnimation = false;
      }
      void toggleKeyFrameAnimation(){
         isKeyFrameAnimation = (isKeyFrameAnimation)? 0: 1;
         if(isKeyFrameAnimation){
            command->isKeyFrameAnimation = true;
         } else {
            command->isKeyFrameAnimation = false;
         }
      }
      int getTimeStep(){
         return command->param.timeStep;
      }
};

	}
}

#endif
