#ifndef __ARGUMENT_H__
#define __ARGUMENT_H__

#include <kvs/CommandLine>

class Argument : public kvs::CommandLine
{
public:

    std::string input_data;
    kvs::TransferFunction transfer_function;
    size_t subpixel_level;
    size_t repeat_level;
    float sampling_step;
    bool dump;
    char sampling_method;
    std::string hexahedra_dir;
    std::string quadratic_tetrahedra_dir;
    bool pyramid;
    bool test_volume;
    size_t component_Id;
    bool GT5D;
    bool GT5D_full;
    bool HTTR;

    Argument( int argc, char** argv ):
        kvs::CommandLine( argc, argv )
    {
        // Add help option (generate help message automatically).
        add_help_option();

        // Add input value.
        add_value( "input file file name.",false );
        add_option( "t","transfer_function",1,false );
        add_option( "s","subpixel_level",1,false );
        add_option( "r","repeat_level",1,false );
        add_option( "step","sampling_step",1,false );
        add_option( "c", "component_Id", 1, false);
        add_option( "S","sampling method",1,false );
        add_option( "d","DEBUG:dump result",0,false );
        add_option( "H","DEBUG:read Hexahedra data dir",1,false );
        add_option( "qt","DEBUG:read Quadratic Tetrahedra data dir",1,false );
        add_option( "p","DEBUG:generate pyramid data",0,false );
        add_option( "V","DEBUG:test volume calculation",0,false );
        add_option( "GT5D","GT5D data",0, false );
        add_option( "GT5D_full","GT5D full torus data",0, false );
        add_option( "HTTR","HTTR data",0, false );

        if ( !this->parse() ) exit( EXIT_FAILURE );

        subpixel_level = 1;
        repeat_level = 4;
        sampling_step = 0.5f;
        component_Id = 0;
        dump = false;
        sampling_method = 'u';
        hexahedra_dir = "";
        quadratic_tetrahedra_dir = "";
        pyramid = false;
        test_volume = false;
        GT5D = false;
        GT5D_full = false;
        HTTR = false;

        if( this->hasOption("t") ) transfer_function.read( this->optionValue<std::string>("t") );
        else transfer_function.create( 256 );
        if( this->hasOption("s") ) subpixel_level = this->optionValue<size_t>("s");
        if( this->hasOption("r") ) repeat_level = this->optionValue<size_t>("r");
        if( this->hasOption("step") ) sampling_step = this->optionValue<float>("step");
        if( this->hasOption("c") ) component_Id = this->optionValue<size_t>("compomentId");
        if( this->hasOption("d") ) dump = true;
        if( this->hasOption("S") ) sampling_method = this->optionValue<char>("S");
        if( this->hasOption("H") ) hexahedra_dir = this->optionValue<std::string>("H");
        if( this->hasOption("qt") ) quadratic_tetrahedra_dir = this->optionValue<std::string>("qt");
        if( this->hasOption("p") ) pyramid = true;
        if( this->hasOption("V") ) test_volume = true;
        if( this->hasOption("GT5D") ) GT5D = true;
        if( this->hasOption("GT5D_full") ) GT5D_full = true;
        if( this->hasOption("HTTR") ) HTTR = true;

        input_data = this->value<std::string>();
    }
};

#endif
