#ifndef __GENERATE_VOLUME_DATA_H__
#define __GENERATE_VOLUME_DATA_H__

#include <kvs/UnstructuredVolumeObject>

extern "C" {
void gen_quadratic_hexahedra_data(
        float* coords,
        unsigned int* connections,
        float* values);

void gen_pyramid_data(
        float* coords,
        unsigned int* connections,
        float* values);
}

class GenerateVolumeData
{
public:

    enum CellType
    {
        QuadraticHexahedra,
        Pyramid
    };

    kvs::UnstructuredVolumeObject* m_volume;

public:

    GenerateVolumeData( const size_t resolution )
        {
            m_volume = new kvs::UnstructuredVolumeObject( );
        }

    ~GenerateVolumeData( void )
        {
            delete m_volume;
        }

    void quadraticHexahedra( const size_t resolution )
        {
            kvs::VolumeObjectBase::CellType cell_type =
                kvs::VolumeObjectBase::QuadraticHexahedra;
            const size_t lsize = resolution - 1;
            const size_t ncells = lsize * lsize * lsize;
            const size_t nnodes = ncells * 20;
            kvs::ValueArray<float> coords( nnodes * 3 );
            kvs::ValueArray<unsigned int> connections( nnodes );
            kvs::ValueArray<float> values( nnodes );

            gen_quadratic_hexahedra_data(coords.pointer(), connections.pointer(), values.pointer());

            m_volume->setCellType( cell_type );
            m_volume->setNNodes( nnodes );
            m_volume->setNCells( ncells );
            m_volume->setVeclen( 1 );
            m_volume->setCoords( coords );
            m_volume->setConnections( connections );
            m_volume->setValues( kvs::AnyValueArray( values ) );
            m_volume->updateMinMaxCoords();
            m_volume->updateMinMaxValues();
        }

    void pyramid( const size_t resolution )
        {
            kvs::VolumeObjectBase::CellType cell_type =
                kvs::VolumeObjectBase::Pyramid;
            const size_t lsize = resolution - 1;
            const size_t ncells = lsize * lsize * lsize * 6;
            const size_t nnodes = ncells * 5;
            kvs::ValueArray<float> coords( nnodes * 3 );
            kvs::ValueArray<unsigned int> connections( nnodes );
            kvs::ValueArray<float> values( nnodes );

            gen_pyramid_data(coords.pointer(), connections.pointer(), values.pointer());
            m_volume->setCellType( cell_type );
            m_volume->setNNodes( nnodes );
            m_volume->setNCells( ncells );
            m_volume->setVeclen( 1 );
            m_volume->setCoords( coords );
            m_volume->setConnections( connections );
            m_volume->setValues( kvs::AnyValueArray( values ) );
            m_volume->updateMinMaxCoords();
            m_volume->updateMinMaxValues();
        }
};

#endif
