#ifndef QUADRATIC_HEXAHEDRA_CUH
#define QUADRATIC_HEXAHEDRA_CUH


__device__ static void quadratic_hexahedra_interpolation_func(
        float* point,
        float* interpolation_func)
{
    const float x = point[0];
    const float y = point[1];
    const float z = point[2];

    const float xy = x * y;
    const float yz = y * z;
    const float zx = z * x;

    const float xyz = xy * z;

    interpolation_func[ 0] = ( 1 - x )*( 1 - y )*z*( - 1 - 2*x - 2*y + 2*z );
    interpolation_func[ 1] = x*( 1 - y )*z*( - 3 + 2*x - 2*y + 2*z );
    interpolation_func[ 2] = xyz*( - 5 + 2*x + 2*y + 2*z );
    interpolation_func[ 3] = ( 1 - x )*yz*( - 3 - 2*x + 2*y + 2*z );
    interpolation_func[ 4] = ( 1 - x )*( 1 - y )*( 1 - z )*( 1 - 2*x - 2*y - 2*z );
    interpolation_func[ 5] = x*( 1 - y )*( 1 - z )*( - 1 + 2*x - 2*y - 2*z );
    interpolation_func[ 6] = xy*( 1 - z )*( - 3 + 2*x + 2*y - 2*z );
    interpolation_func[ 7] = ( 1 - x )*y*( 1 - z )*( - 1 - 2*x + 2*y - 2*z );
    interpolation_func[ 8] = 4*zx*( 1 - x )*( 1 - y );
    interpolation_func[ 9] = 4*xyz*( 1 - y );
    interpolation_func[10] = 4*xyz*( 1 - x );
    interpolation_func[11] = ( 1 - x )*4*yz*( 1 - y );
    interpolation_func[12] = 4*x*( 1 - x )*( 1 - y )*( 1 - z );
    interpolation_func[13] = 4*xy*( 1 - y )*( 1 - z );
    interpolation_func[14] = 4*xy*( 1 - x )*( 1 - z );
    interpolation_func[15] = ( 1 - x )*4*y*( 1 - y )*( 1 - z );
    interpolation_func[16] = ( 1 - x )*( 1 - y )*4*z*( 1 - z );
    interpolation_func[17] = ( 1 - y )*4*zx*( 1 - z );
    interpolation_func[18] = 4*xyz*( 1 - z );
    interpolation_func[19] = ( 1 - x )*4*yz*( 1 - z );
}


__device__ static void quadratic_hexahedra_differential_func(
        float* point,
        float* differential_func)
{
    const float x = point[0];
    const float y = point[1];
    const float z = point[2];

    // dNdx
    differential_func[ 0] =  -(1-y)*z*(2*z-2*y-2*x-1)-2*(1-x)*(1-y)*z;
    differential_func[ 1] =  (1-y)*z*(2*z-2*y+2*x-3)+2*x*(1-y)*z;
    differential_func[ 2] =  y*z*(2*z+2*y+2*x-5)+2*x*y*z;
    differential_func[ 3] =  -y*z*(2*z+2*y-2*x-3)-2*(1-x)*y*z;
    differential_func[ 4] =  -(1-y)*(-2*z-2*y-2*x+1)*(1-z)-2*(1-x)*(1-y)*(1-z);
    differential_func[ 5] =  (1-y)*(-2*z-2*y+2*x-1)*(1-z)+2*x*(1-y)*(1-z);
    differential_func[ 6] =  y*(-2*z+2*y+2*x-3)*(1-z)+2*x*y*(1-z);
    differential_func[ 7] =  -y*(-2*z+2*y-2*x-1)*(1-z)-2*(1-x)*y*(1-z);
    differential_func[ 8] =  4*(1-x)*(1-y)*z-4*x*(1-y)*z;
    differential_func[ 9] =  4*(1-y)*y*z;
    differential_func[10] =  4*(1-x)*y*z-4*x*y*z;
    differential_func[11] =  -4*(1-y)*y*z;
    differential_func[12] =  4*(1-x)*(1-y)*(1-z)-4*x*(1-y)*(1-z);
    differential_func[13] =  4*(1-y)*y*(1-z);
    differential_func[14] =  4*(1-x)*y*(1-z)-4*x*y*(1-z);
    differential_func[15] =  -4*(1-y)*y*(1-z);
    differential_func[16] =  -4*(1-y)*(1-z)*z;
    differential_func[17] =  4*(1-y)*(1-z)*z;
    differential_func[18] =  4*y*(1-z)*z;
    differential_func[19] =  -4*y*(1-z)*z;

    // dNdy
    differential_func[20] =  -(1-x)*z*(2*z-2*y-2*x-1)-2*(1-x)*(1-y)*z;
    differential_func[21] =  -x*z*(2*z-2*y+2*x-3)-2*x*(1-y)*z;
    differential_func[22] =  x*z*(2*z+2*y+2*x-5)+2*x*y*z;
    differential_func[23] =  (1-x)*z*(2*z+2*y-2*x-3)+2*(1-x)*y*z;
    differential_func[24] =  -(1-x)*(-2*z-2*y-2*x+1)*(1-z)-2*(1-x)*(1-y)*(1-z);
    differential_func[25] =  -x*(-2*z-2*y+2*x-1)*(1-z)-2*x*(1-y)*(1-z);
    differential_func[26] =  x*(-2*z+2*y+2*x-3)*(1-z)+2*x*y*(1-z);
    differential_func[27] =  (1-x)*(-2*z+2*y-2*x-1)*(1-z)+2*(1-x)*y*(1-z);
    differential_func[28] =  -4*(1-x)*x*z;
    differential_func[29] =  4*x*(1-y)*z-4*x*y*z;
    differential_func[30] =  4*(1-x)*x*z;
    differential_func[31] =  4*(1-x)*(1-y)*z-4*(1-x)*y*z;
    differential_func[32] =  -4*(1-x)*x*(1-z);
    differential_func[33] =  4*x*(1-y)*(1-z)-4*x*y*(1-z);
    differential_func[34] =  4*(1-x)*x*(1-z);
    differential_func[35] =  4*(1-x)*(1-y)*(1-z)-4*(1-x)*y*(1-z);
    differential_func[36] =  -4*(1-x)*(1-z)*z;
    differential_func[37] =  -4*x*(1-z)*z;
    differential_func[38] =  4*x*(1-z)*z;
    differential_func[39] =  4*(1-x)*(1-z)*z;

    // dNdz
    differential_func[40] =  (1-x)*(1-y)*(2*z-2*y-2*x-1)+2*(1-x)*(1-y)*z;
    differential_func[41] =  x*(1-y)*(2*z-2*y+2*x-3)+2*x*(1-y)*z;
    differential_func[42] =  x*y*(2*z+2*y+2*x-5)+2*x*y*z;
    differential_func[43] =  (1-x)*y*(2*z+2*y-2*x-3)+2*(1-x)*y*z;
    differential_func[44] =  -2*(1-x)*(1-y)*(1-z)-(1-x)*(1-y)*(-2*z-2*y-2*x+1);
    differential_func[45] =  -2*x*(1-y)*(1-z)-x*(1-y)*(-2*z-2*y+2*x-1);
    differential_func[46] =  -2*x*y*(1-z)-x*y*(-2*z+2*y+2*x-3);
    differential_func[47] =  -2*(1-x)*y*(1-z)-(1-x)*y*(-2*z+2*y-2*x-1);
    differential_func[48] =  4*(1-x)*x*(1-y);
    differential_func[49] =  4*x*(1-y)*y;
    differential_func[50] =  4*(1-x)*x*y;
    differential_func[51] =  4*(1-x)*(1-y)*y;
    differential_func[52] =  -4*(1-x)*x*(1-y);
    differential_func[53] =  -4*x*(1-y)*y;
    differential_func[54] =  -4*(1-x)*x*y;
    differential_func[55] =  -4*(1-x)*(1-y)*y;
    differential_func[56] =  4*(1-x)*(1-y)*(1-z)-4*(1-x)*(1-y)*z;
    differential_func[57] =  4*x*(1-y)*(1-z)-4*x*(1-y)*z;
    differential_func[58] =  4*x*y*(1-z)-4*x*y*z;
    differential_func[59] =  4*(1-x)*y*(1-z)-4*(1-x)*y*z;
}

/*** debug ***/ 
__device__ static float quadratic_hexahedra_density
(
 float* density_map,
 unsigned int resolution,
 float  normalize_factor,
 float  min_value,
 float* vertices,
 float* scalars,
 int    ndiv,
 float* x,
 float* w
 )
{
  float  density;
  float  v[3];
  float  scalar;
  float  interpolation_func[20];
  float  differential_func[60];
  int    nnodes=20;
  float  A,B;
  float  J[3][3],jac;

  A = 0.5;
  B = 0.5;
  density = 0.0;
  for(int k=0;k<ndiv;k++){
    v[2] = B+A*x[k];
    for(int j=0;j<ndiv;j++){
      v[1] = B+A*x[j];
      for(int i=0;i<ndiv;i++){
	v[0] = B+A*x[i];
	// setLocalPoint
	quadratic_hexahedra_interpolation_func(v,interpolation_func);
	quadratic_hexahedra_differential_func(v,differential_func);
// 	jacobi_matrix(nnodes, vertices, differential_func, J);
//  	jac = fabsf(determinant(J));
 	jac = 1.0;
	scalar = 0.0;
	for(int m=0;m<nnodes;m++){
	  scalar += scalars[m]*interpolation_func[m];
	}
	density += 
	  w[i]*w[j]*w[k]
	  *jac*calculate_density(
				 density_map,
				 resolution,
				 normalize_factor,
				 min_value,
				 scalar);
      }
    }
  }
  return fabsf(A*A*A*density);
}
/*************/

__device__ static float quadratic_hexahedra_volume(
        float* vertices,
        float* interpolation_func,
        float* differential_func)
{
    const size_t resolution = 3;
    const float sampling_length = 1.0f / (float)resolution;
    const float adjustment = sampling_length * 0.5f;

    float sampling_position[3] = {
        -adjustment,
        -adjustment,
        -adjustment
    };
    float sum_metric = 0;
    float J[3][3];

    for (size_t k = 0; k < resolution; k++) {
        sampling_position[2] += sampling_length;
        for (size_t j = 0; j < resolution; j++) {
            sampling_position[1] += sampling_length;
            for (size_t i = 0; i < resolution; i++) {
                sampling_position[0] += sampling_length;

                // setLocalPoint
                quadratic_hexahedra_interpolation_func(
                        sampling_position, interpolation_func);
                quadratic_hexahedra_differential_func(
                        sampling_position, differential_func);

                // Jacobi Matrix
                jacobi_matrix(20, vertices, differential_func, J);

                // determiant
                float metric_element = determinant(J);
                sum_metric += fabsf(metric_element);
            }
            sampling_position[0] = -adjustment;
        }
        sampling_position[1] = -adjustment;
    }

    return sum_metric / (resolution * resolution * resolution);
}

__device__ static void quadratic_hexahedra_random_sampling(
        float* vertices,
        float* interpolation_func,
        float* differential_func,
	/*** debug ***/
	float s,
	float t,
	float u,
//         tinymt32_status_t* tinymt32,
	/*************/
        float* coord)
    {
    const int nnodes = 20;
    float point[3];
    /*** debug ***/
    point[0] = s;
    point[1] = t;
    point[2] = u;
//     point[0] = tinymt32_single(tinymt32);
//     point[1] = tinymt32_single(tinymt32);
//     point[2] = tinymt32_single(tinymt32);
    /*************/

    // setLocalPoint
    quadratic_hexahedra_interpolation_func(point, interpolation_func);
    quadratic_hexahedra_differential_func(point, differential_func);

    // transformLocalToGlobal
    coord[0] = 0;
    for (int i = 0; i < nnodes; i++) coord[0] += interpolation_func[i] * vertices[i * 3];

    coord[1] = 0;
    for (int i = 0; i < nnodes; i++) coord[1] += interpolation_func[i] * vertices[i * 3 + 1];

    coord[2] = 0;
    for (int i = 0; i < nnodes; i++) coord[2] += interpolation_func[i] * vertices[i * 3 + 2];
}

#endif

