#include "ExtendedTransferFunction.h"

ExtendedTransferFunction::ExtendedTransferFunction()
{
}

ExtendedTransferFunction::~ExtendedTransferFunction( void )
{
}

ExtendedTransferFunction::ExtendedTransferFunction( const TransferFunction& tf )
    : TransferFunction( tf )
{
}

ExtendedTransferFunction::ExtendedTransferFunction( std::string rfstr,
                                                    std::string gfstr,
                                                    std::string bfstr,
                                                    std::string ofstr,
                                                    const size_t      resolution,
                                                    const float       min_value,
                                                    const float       max_value )
    : TransferFunction( resolution )
{
    kvs::ColorMap*   c = new kvs::ColorMap( resolution,min_value,max_value );
    kvs::OpacityMap* a = new kvs::OpacityMap( resolution,min_value,max_value );

    FuncParser::Variables vars;
    FuncParser::Variable var_x;
    FuncParser::Function rf,gf,bf,of;

    var_x.tag( "x" );
    vars.push_back( var_x );

    FuncParser::FunctionParser rf_parse( rfstr, rfstr.size() + 1 );
    FuncParser::FunctionParser gf_parse( gfstr, gfstr.size() + 1 );
    FuncParser::FunctionParser bf_parse( bfstr, bfstr.size() + 1 );
    FuncParser::FunctionParser of_parse( ofstr, ofstr.size() + 1 );
    FuncParser::FunctionParser::Error err_r = rf_parse.express( rf, vars );
    FuncParser::FunctionParser::Error err_g = gf_parse.express( gf, vars );
    FuncParser::FunctionParser::Error err_b = bf_parse.express( bf, vars );
    FuncParser::FunctionParser::Error err_o = of_parse.express( of, vars );

    const float stride = ( max_value - min_value ) / ( resolution - 1 );
    float x = min_value;
    for ( size_t i = 0; i < resolution; ++i, x+= stride )
    {
        kvs::UInt8 r,g,b;
        float opacity;

        var_x = x;
        r = rf.eval();
        g = gf.eval();
        b = bf.eval();
        opacity = of.eval();

        kvs::RGBColor color( r,g,b );
        c->addPoint( x, color );
        a->addPoint( x, opacity );
    }
    c->create();
    a->create();

    this->setColorMap( *c );
    this->setOpacityMap( *a );
    this->setRange( min_value, max_value );

    delete c,a;
}
