#include <iostream>
#include <fstream>

#include "FilterInfo.h"
#include "endian2.h"

int FilterInfo::LoadPFI( std::string& filename )
{
    std::ifstream fin( filename.c_str(), std::ios::in | std::ios::binary );

    fin.read( ( char* )&numNodes, sizeof( int32_t ) );
    fin.read( ( char* )&numElements, sizeof( int32_t ) );
    fin.read( ( char* )&elemType, sizeof( int32_t ) );
    fin.read( ( char* )&fileType, sizeof( int32_t ) );
    fin.read( ( char* )&numFiles, sizeof( int32_t ) );
    fin.read( ( char* )&numIngredients, sizeof( int32_t ) );
    fin.read( ( char* )&staSteps, sizeof( int32_t ) );
    fin.read( ( char* )&endSteps, sizeof( int32_t ) );
    fin.read( ( char* )&numSubVolumes, sizeof( int32_t ) );

    endian2::ltoh( numNodes );
    endian2::ltoh( numElements );
    endian2::ltoh( elemType );
    endian2::ltoh( fileType );
    endian2::ltoh( numFiles );
    endian2::ltoh( numIngredients );
    endian2::ltoh( staSteps );
    endian2::ltoh( endSteps );
    endian2::ltoh( numSubVolumes );
    numSteps = endSteps - staSteps + 1;

    float x_min, y_min, z_min;
    float x_max, y_max, z_max;
    fin.read( ( char* )&x_min, sizeof( float ) );
    fin.read( ( char* )&y_min, sizeof( float ) );
    fin.read( ( char* )&z_min, sizeof( float ) );
    fin.read( ( char* )&x_max, sizeof( float ) );
    fin.read( ( char* )&y_max, sizeof( float ) );
    fin.read( ( char* )&z_max, sizeof( float ) );
    endian2::ltoh( x_min );
    endian2::ltoh( y_min );
    endian2::ltoh( z_min );
    endian2::ltoh( x_max );
    endian2::ltoh( y_max );
    endian2::ltoh( z_max );
    minObjectCoord.set( x_min, y_min, z_min );
    maxObjectCoord.set( x_max, y_max, z_max );

    int skipsize = 0;
    skipsize += ( numSubVolumes + numSubVolumes ) * sizeof(int);
    fin.seekg( skipsize, std::ios::cur );

    minSubVolumeCoord.resize( numSubVolumes );
    maxSubVolumeCoord.resize( numSubVolumes );
    for (int32_t vl = 0; vl < numSubVolumes; vl++) {
        float sub_x_min, sub_y_min, sub_z_min;
        float sub_x_max, sub_y_max, sub_z_max;
        fin.read( ( char* )&sub_x_min, sizeof( float ) );
        fin.read( ( char* )&sub_y_min, sizeof( float ) );
        fin.read( ( char* )&sub_z_min, sizeof( float ) );
        fin.read( ( char* )&sub_x_max, sizeof( float ) );
        fin.read( ( char* )&sub_y_max, sizeof( float ) );
        fin.read( ( char* )&sub_z_max, sizeof( float ) );
        endian2::ltoh( sub_x_min );
        endian2::ltoh( sub_y_min );
        endian2::ltoh( sub_z_min );
        endian2::ltoh( sub_x_max );
        endian2::ltoh( sub_y_max );
        endian2::ltoh( sub_z_max );
        minSubVolumeCoord[vl].set( sub_x_min, sub_y_min, sub_z_min );
        maxSubVolumeCoord[vl].set( sub_x_max, sub_y_max, sub_z_max );
    }

    ingredStep.clear();
    for (int32_t s = 0; s < numSteps; s++) {
        ingredStep.push_back( ingredientsStep() );
        ingredStep[s].ingred.clear();
        for (int32_t i = 0; i < numIngredients; i++ ) {
            float min,max;
            fin.read( ( char* )&min, sizeof( float ) );
            fin.read( ( char* )&max, sizeof( float ) );
            endian2::ltoh( min );
            endian2::ltoh( max );

            ingredStep[s].ingred.push_back(ingredientsMinMax());
            ingredStep[s].ingred[i].min = min;
            ingredStep[s].ingred[i].max = max;
        }
    }

    minValue = ingredStep[0].ingred[0].min;
    maxValue = ingredStep[0].ingred[0].max;
    for (int32_t s = 0; s < numSteps; s++) {
        for (int32_t i = 0; i < numIngredients; i++ ) {
            float min = ingredStep[s].ingred[i].min;
            float max = ingredStep[s].ingred[i].max;
            if( min < minValue ) minValue = min;
            if( max > maxValue ) maxValue = max;
        }
    }

    fin.close();

    /*
    std::cout << "numNodes:      " << numNodes       << std::endl;
    std::cout << "numElements:   " << numElements    << std::endl;
    std::cout << "elemType:      " << elemType       << std::endl;
    std::cout << "fileType:      " << fileType       << std::endl;
    std::cout << "numFiles:      " << numFiles       << std::endl;
    std::cout << "numIngredients:" << numIngredients << std::endl;
    std::cout << "numSteps:      " << numSteps       << std::endl;
    std::cout << "numSubVolumes: " << numSubVolumes  << std::endl;
    */

    return 0;
}


