/****************************************************************************/
/**
 *  @file GTimestepObjectUnstructuredVolume.h
 */
/****************************************************************************/
#ifndef KVS__GTIMESTEP_OBJECT_UNSTRUCTURED_VOLUME_H_INCLUDE
#define KVS__GTIMESTEP_OBJECT_UNSTRUCTURED_VOLUME_H_INCLUDE

#include "FileFormatBase.h"
#include "AnyValueArray.h"
#include "ValueArray.h"
#include "Type.h"
#include "Vector3.h"
#include <string>
#include "ObjectTag.h"


namespace kvs
{
    
/*===========================================================================*/
/**
 *  @brief  GTimestep unstructured volume object format.
 */
/*===========================================================================*/
class GTimestepObjectUnstructuredVolume : public kvs::FileFormatBase
    {
        kvsClassName( kvs::GTimestepObjectUnstructuredVolume );
            
public:

    enum WritingDataType
    {
        Ascii = 0,
        ExternalAscii,
        ExternalBinary
    };

protected:

    std::string                  m_cell_type;     ///< cell type
    size_t                       m_veclen;        ///< vector length
    size_t                       m_nnodes;        ///< number of nodes
    size_t                       m_ncells;        ///< number of cells
    kvs::AnyValueArray           m_values;        ///< field value array
    kvs::ValueArray<kvs::Real32> m_coords;        ///< coordinate value array
    kvs::ValueArray<kvs::UInt32> m_connections;   ///< connection id array

public:

    GTimestepObjectUnstructuredVolume( void );

    GTimestepObjectUnstructuredVolume( const std::string& filename, int st, int vl );

    virtual ~GTimestepObjectUnstructuredVolume( void );

public:

    const std::string cellType( void ) const;

    const size_t veclen( void ) const;

    const size_t nnodes( void ) const;

    const size_t ncells( void ) const;

    const kvs::AnyValueArray& values( void ) const;

    const kvs::ValueArray<kvs::Real32>& coords( void ) const;

    const kvs::ValueArray<kvs::UInt32>& connections( void ) const;

public:

    void setCellType( const std::string cellType );

    void setVeclen( const size_t veclen );

    void setNNodes( const size_t nnodes );

    void setNCells( const size_t ncells );

    void setValues( const kvs::AnyValueArray& values );

    void setCoords( const kvs::ValueArray<kvs::Real32>& coords );

    void setConnections( const kvs::ValueArray<kvs::UInt32>& connections );

public:

    const bool read( const std::string& filename );
    
    const bool write( const std::string& filename );
    
  };
} // end of namespace kvs
#endif // KVS__GTIMESTEP_OBJECT_UNSTRUCTURED_VOLUME_H_INCLUDE
