#if defined JPV_CLIENT

#include "ParticleTransferClient.h"

#include <iostream>
#include <sstream>
#include <cstring>
#include <fstream>

#if defined WIN32
#else
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#endif

jpv::ParticleTransferClient::ParticleTransferClient( std::string _host,int _port )
    : hostname( _host ), port( _port )
{
}

jpv::ParticleTransferClient::~ParticleTransferClient( void )
{
}

int jpv::ParticleTransferClient::initClient( void )
{
    struct hostent* servhost;
    struct sockaddr_in dest;
    int flg = 0;
    memset( &dest, 0, sizeof( dest ) );

#if defined WIN32
    WSADATA data;
    WSAStartup( MAKEWORD( 2,0 ), &data );
#endif
    servhost = gethostbyname( hostname.c_str() );
    if ( servhost == NULL )
    {
        std::cout << "resolv error: " << hostname << std::endl;
        return -1;
    }

    dest.sin_port = htons( port );
    dest.sin_family = AF_INET;
    memcpy( &dest.sin_addr, servhost->h_addr, servhost->h_length );

    sock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
    if( connect( sock, ( struct sockaddr* ) &dest, sizeof( dest ) ) == -1 )
    {
        std::cout << "connection error" << std::endl;
        return -1;
    }
    return 0;
}

int jpv::ParticleTransferClient::termClient( void )
{

#if defined WIN32
    shutdown( sock,SD_BOTH );
    std::cout << "shutdown connection" << std::endl;
    closesocket( sock );
    std::cout << "close socket" << std::endl;
    WSACleanup();
#else
    shutdown( sock,SHUT_RDWR );
    std::cout << "shutdown connection" << std::endl;
    close( sock );
    std::cout << "close socket" << std::endl;
#endif
    return 0;
}

int jpv::ParticleTransferClient::sendMessage( ParticleTransferClientMessage& message )
{
    int size = message.byteSize();
    char* buf = new char [size];

    std::cout << "Send Client Message Size = " << size << std::endl;

    message.pack( buf );
    send( sock, buf, size, 0 );

#ifdef _DEBUG
    std::ofstream output;
    output.open( "debug_client.send" );
    output.write( buf, size );
    output.flush();
    output.close();
#endif

    delete[] buf;
    return 0;
}

int jpv::ParticleTransferClient::recvMessage( ParticleTransferServerMessage& message )
{
    // T[o[bZ[W̎M
    const size_t hSize = sizeof( message.header ) + sizeof( message.messageSize );
    char hsbuf[hSize];
    char* buf;
    std::stringstream ss;
    int recvSize( 0 );
    size_t mSize,rSize;

    message.messageSize = 0;
    for ( rSize = 0; rSize < hSize; rSize += recvSize )
    {
        recvSize = recv( sock,hsbuf,hSize-rSize,0 );
        ss.write( hsbuf,recvSize );
    }
    ss.seekg( sizeof( message.header ) );
    ss.read( reinterpret_cast<char*>( &message.messageSize ), sizeof( message.messageSize ) );
    std::cout << "Receive Server Message Size = " << message.messageSize << std::endl;

    mSize = message.messageSize-hSize;
    buf = new char[mSize];
    for ( rSize = 0; rSize < mSize; rSize += recvSize )
    {
        recvSize = recv( sock,buf,mSize-rSize,0 );
        ss.write( buf,recvSize );
    }
    delete[] buf;

    message.unpack_message( ss.str().c_str() );
    std::cout << "Receive Particle Size = " << message.numParticle << std::endl;

    if( message.numParticle > 0 )
    {
        const size_t pSize = ( 12+12+3 ) * message.numParticle;
        buf = new char[pSize];
        for ( rSize = 0; rSize < pSize; rSize += recvSize )
        {
            recvSize = recv( sock,buf,pSize-rSize,0 );
            ss.write( buf,recvSize );
        }
        message.unpack_particles( ss.str().c_str() );
        delete[] buf;
    }

#ifdef _DEBUG
    std::ofstream output;
    output.open( "debug_client.recv" );
//    output.write( ss.str().c_str(), message.messageSize + pSize );
    output.flush();
    output.close();
#endif

    return 0;
}
#endif

