#include "PointObjectGenerator.h"
//#include <sys/time.h>
#include "TransferFunction.h"
#include "UnstructuredVolumeObject.h"
#include "UnstructuredVolumeImporter.h"
#include "CellByCellUniformSampling.h"
#include "CellByCellRejectionSampling.h"
#include "CellByCellMetropolisSampling.h"
//#include "CellByCellLayeredSampling.h"
#include "Camera.h"
#include "CellByCellUniformSampling.h"
#include "CellByCellRejectionSampling.h"
#include "CellByCellMetropolisSampling.h"
#include "TestVolume.h"
#include "FrontSTRFileReader.h"
#include "DistributedUnstructuredVolumeImporter.h"
#include "AVSUcd.h"
#include "GenerateVolumeData.h"
#include "point_object_summation.h"
#include "common.h"
#include "Profiler.h"
#include "ValueArray.h"

#include "Argument.h"
#ifdef KMATH
#include <kmath_random.h>
#endif

#include "timer_simple.h"

using namespace kvs;
#ifdef KMATH
extern KMATH_Random km_random;
#endif

void PointObjectGenerator::createTestObject( const size_t howMany )
{
    delete object;
    kvs::ValueArray<float> points( howMany*3 );
    double* rv = new double[howMany*3*2];
#ifdef KMATH
    #pragma omp critical(random)
    {
        km_random.get( rv,howMany*3*2 );
    }
    for( size_t i = 0; i < howMany*3*2; ++i )
    {
        rv[i] -= 1.0;
    }
#endif
    for( size_t i = 0; i < howMany; ++i )
    {
#ifndef KMATH
        points[i*3] = ( rand() % 1000 ) / 1000.0f * 2.0f;
        points[i*3+1] = ( rand() % 1000 ) / 1000.0f * 2.0f;
        points[i*3+2] = ( rand() % 1000 ) / 1000.0f * 2.0f;
#else
        points[i*3]   = ( static_cast<int>( rv[i*3  ] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
        points[i*3+1] = ( static_cast<int>( rv[i*3+1] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
        points[i*3+2] = ( static_cast<int>( rv[i*3+2] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
#endif
    }
    kvs::ValueArray<kvs::UInt8> colors( howMany*3 );
    for( size_t i = 0; i < howMany; ++i )
    {
#ifndef KMATH
        colors[i*3] = ( rand() % 1000 ) / 1000.0f * 255;
        colors[i*3+1] = ( rand() % 1000 ) / 1000.0f * 255;
        colors[i*3+2] = ( rand() % 1000 ) / 1000.0f * 255;;
#else
        colors[i*3]   = ( static_cast<int>( rv[howMany+i*3  ] * RAND_MAX ) % 1000 ) / 1000.0f * 255;
        colors[i*3+1] = ( static_cast<int>( rv[howMany+i*3+1] * RAND_MAX ) % 1000 ) / 1000.0f * 255;
        colors[i*3+2] = ( static_cast<int>( rv[howMany+i*3+2] * RAND_MAX ) % 1000 ) / 1000.0f * 255;;
#endif
    }
    delete[] rv;
    object = new kvs::PointObject( points, colors, 30 );
}

void PointObjectGenerator::createHexahedraPointObject( Argument& param,Camera& camera, size_t subpixel_level, float sampling_step )
{
    delete object;
    const int DATA_NUM = 16;

    size_t total_particles = 0;
    kvs::UnstructuredVolumeObject* volume;
    kvs::PointObject* p_object[DATA_NUM];

    for ( int i = 0; i < DATA_NUM; i++ )
    {
        char filename[128];
        sprintf( filename, "P00%02d/7k_P00%02d.kvsml", i+1, i+1 );
        std::string filepath = param.hexahedra_dir + '/' + filename;
        std::cout << filepath << std::endl;
        volume = new kvs::UnstructuredVolumeImporter( filepath );
        std::cout << *volume << std::endl;

        p_object[i] = sampling( param, camera, volume, subpixel_level, sampling_step );

        delete volume;
        total_particles += p_object[i]->nvertices();
    }
    if ( !param.test_volume )
    {
        printf( "total %zu\n", total_particles );
    }

    object = new kvs::PointObjectSummation( p_object, DATA_NUM, total_particles );

    for ( int i = 0; i < DATA_NUM; i++ )
    {
        delete p_object[i];
    }
}

void PointObjectGenerator::createPyramidPointObject( Argument& param,Camera& camera, size_t subpixel_level, float sampling_step )
{
    delete object;
    kvs::UnstructuredVolumeObject* volume;
    GenerateVolumeData gen_vol( 128 );

    std::cout << "gen pyramid data." << std::endl;
    gen_vol.pyramid( 128 );
    volume = gen_vol.m_volume;

    std::cout << *volume << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
}

void PointObjectGenerator::createQuadraticTetrahedra( Argument& param, Camera& camera, size_t subpixel_level, float sampling_step )
{
    delete object;
    kvs::FrontSTRFileReader* reader = new kvs::FrontSTRFileReader(
        param.quadratic_tetrahedra_dir.c_str(),
        0,
        1,
        kvs::FrontSTRFileReader::Skip,
        kvs::UnstructuredVolumeObject::QuadraticTetrahedra );

    kvs::UnstructuredVolumeObject* volume = new kvs::DistributedUnstructuredVolumeImporter( reader );

    delete reader;

    std::cout << *volume << std::endl;
    object = sampling( param, camera, volume, subpixel_level, sampling_step );
}

void PointObjectGenerator::createQuadraticHexahedra( Argument& param,Camera& camera, size_t subpixel_level, float sampling_step )
{
    delete object;
    kvs::UnstructuredVolumeObject* volume;
    GenerateVolumeData gen_vol( 128 );

    std::cout << "gen quadratic hexahedra data." << std::endl;
    gen_vol.quadraticHexahedra( 128 );
    volume = gen_vol.m_volume;

    std::cout << *volume << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
}

void PointObjectGenerator::createHTTR(
    Argument& param, Camera& camera, size_t subpixel_level, float sampling_step, const size_t step )
{
    delete object;

    char filename[256];
    sprintf( filename,"%s/binary0%03d.dat", param.input_data.c_str(), step );
    //sprintf(filename,"%s/divide0%03d-04639.dat", param.input_data.c_str(), step);
    std::cout << filename << std::endl;
    kvs::visclient::Profiler::get()->start( "file read" );
    kvs::UnstructuredVolumeObject*  volume = read_avsucd( filename, param.component_Id );
    kvs::visclient::Profiler::get()->end( "file read" );

    std::cout << *volume << std::endl;
    kvs::visclient::Profiler::get()->start( "sampling" );
    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    kvs::visclient::Profiler::get()->end( "sampling" );
    delete volume;
}

void PointObjectGenerator::createGT5D(
    Argument& param, Camera& camera, size_t subpixel_level,
    float sampling_step, kvs::UnstructuredVolumeObject* volume )
{
    delete object;
    std::cout<<"start particle generation\n";
    std::cout << *volume << std::endl;
    kvs::visclient::Profiler::get()->start( "totalParticleGen." );
    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    kvs::visclient::Profiler::get()->end( "totalParticleGen." );
    std::cout<<"end particle generation\n";
}

void PointObjectGenerator::createGT5D_full(
    Argument& param, Camera& camera, size_t subpixel_level,
    float sampling_step, kvs::UnstructuredVolumeObject* volume )
{
    delete object;
    kvs::TransferFunction& tf = param.transfer_function;
    kvs::visclient::Profiler::get()->start( "totalParticleGen." );
    object = new kvs::CellByCellUniformSampling
    ( &camera, volume, subpixel_level, sampling_step, tf,
      param.transfunc_synthesizer, param.normal_ingred, param.crap, true );
    kvs::visclient::Profiler::get()->end( "totalParticleGen." );
}
/**************************
    int mpi_size = 1;
    int ts_file_no = -1;
    int ts_file_max_no = 0;

#ifdef ENABLE_MPI
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
    if(mpi_rank == 0){
#endif
        kvs::test::AVSField* field =
            new kvs::test::AVSField( header, coord, variable );
        kvs::UnstructuredVolumeObject* volume = new kvs::UnstructuredVolumeObject;
        for( size_t i = 0; i < GT5D_NUM; i++ )
        {
#ifdef ENABLE_MPI
            if((time_series==0)||(mpi_rank==i%mpi_size)){
                if(time_series==1) MPI_Barrier(MPI_COMM_WORLD);
#endif
                while(field->currentTimeStep() != i){
                    kvs::PointObject* pobject;
                }
                field->progress();
                Import( volume, field, i ,0);
                switch (param.sampling_method) {
                case 'u':
                    std::cout << "Uniform sampling" << std::endl;
                    pobject = new kvs::test::CellByCellUniformSampling(
                        screen.camera(), volume, subpixel_level, sampling_step, tfunc);
                    -->calling();
                    {
                        int sampling_method = MPIController::UNIFORM_SAMPLING;
                        MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                        DistributedUniformSampling::generate_particles_master_gpu(
                            m_coords etc..);
                        -->calling();
                        {
                            DistributedUniformSampling::generate_particles_core_gpu(
                                m_coords etc..);
                        }
                    }
                }
            }
        }
#ifdef ENABLE_MPI
//          if(time_series==1) MPI_Barrier(MPI_COMM_WORLD);
        int exit_command = MPIController::EXIT_PROCESS;
        MPI_Bcast(&exit_command, 1, MPI_INT, 0, MPI_COMM_WORLD);
        for(int i=1;i<mpi_size;i++){
            MPI_Recv(&TIMER[i][0], 100, MPI_REAL, i, 0, MPI_COMM_WORLD,
                     MPI_STATUS_IGNORE);
        }
#endif
        delete volume;
        field->close();
        delete field;
#ifdef ENABLE_MPI
        MPI_Finalize();
#endif
#ifdef ENABLE_MPI
    } // end of if(mpi_rank == 0 )
    else{ // ELSE (mpi_rank !=0)
        init_tinymt();
        if(time_series==1){
            kvs::test::AVSField* field =
                new kvs::test::AVSField( header, coord, variable );
            kvs::UnstructuredVolumeObject* volume = new kvs::UnstructuredVolumeObject;
            ts_file_max_no = ((GT5D_NUM-1)/mpi_size+1) * mpi_size;
            size_t total_particles = 0;
            for (int i = 0; i < ts_file_max_no; i++) {
                if(mpi_rank == i%mpi_size){
                    MPI_Barrier(MPI_COMM_WORLD);
                    ts_file_no = std::min(GT5D_NUM-1,i);
                    while(field->currentTimeStep() != ts_file_no){
                        field->progress();
                    }
                    field->progress();
                    Import( volume, field, ts_file_no, mpi_rank);
                    int sampling_method = 0;
                    MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                    if(sampling_method == MPIController::EXIT_PROCESS){
                        break;
                    }
                    switch(sampling_method){
                    case MPIController::UNIFORM_SAMPLING:
                        DistributedUniformSampling::generate_particles_slave_ts_gpu(vol);
                        -->calling();
                        {
                            DistributedUniformSampling::generate_particles_core_gpu(
                                volume,NULL);
                        }
                    }
                    timer.stop();
                    std::cout<<mpi_rank<<":sampling time:"<<timer.msec()<<std::endl<<std::endl;
                }
            }

        }
        else{ // time_series==0
            for(;;){
                int sampling_method = 0;
                MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                if(sampling_method == MPIController::EXIT_PROCESS){
                    break;
                }
                switch(sampling_method){
                case MPIController::UNIFORM_SAMPLING:
                    kvs::test::DistributedUniformSampling::generate_particles_slave_gpu();
                    -->calling();
                    {
                        DistributedUniformSampling::generate_particles_core_gpu(
                            NULL,NULL);
                    }
                }
            }
            MPI_Send(&TIMER[mpi_rank][0], 100, MPI_REAL, 0, 0, MPI_COMM_WORLD);
        } // end of else( time_series==0)
    } // end of else (mpi_rank!=0)
    MPI_Finalize();
*************************/

void PointObjectGenerator::createFromFile( Argument& param, Camera& camera, size_t subpixel_level, float sampling_step )
{
//FJ_TIMER_KAWAMURA
    TIMER_STA( 260 );
//FJ_TIMER_KAWAMURA

    delete object;
    kvs::UnstructuredVolumeObject* volume;
    volume = new kvs::UnstructuredVolumeImporter( param.input_data );

//FJ_TIMER_KAWAMURA
    TIMER_END( 260 );
//FJ_TIMER_KAWAMURA

    volume->setMinMaxValues( m_fi->minValue, m_fi->maxValue );
    volume->setMinMaxObjectCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );
    volume->setMinMaxExternalCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );

    std::cout << *volume << std::endl;
    std::cout<<"min:"<<volume->minObjectCoord()<<", max:"<<volume->maxObjectCoord()<<std::endl;
    std::cout<<"min:"<<volume->minExternalCoord()<<", max:"<<volume->maxExternalCoord()<<std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    delete volume;
}

void PointObjectGenerator::createFromFile( Argument& param, Camera& camera, size_t subpixel_level, float sampling_step, int st, int vl )
{
    TIMER_STA( 260 );
    delete object;
    kvs::UnstructuredVolumeObject* volume;

    // volume = new kvs::UnstructuredVolumeImporter( param.input_data );
    volume = new kvs::UnstructuredVolumeImporter(param.input_data_base, m_fi->fileType, st, vl);
    
    TIMER_END( 260 );

    volume->setMinMaxValues( m_fi->minValue, m_fi->maxValue );
    volume->setMinMaxObjectCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );
    volume->setMinMaxExternalCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );

    std::cout << *volume << std::endl;
    std::cout<<"min:"<<volume->minObjectCoord()<<", max:"<<volume->maxObjectCoord()<<std::endl;
    std::cout<<"min:"<<volume->minExternalCoord()<<", max:"<<volume->maxExternalCoord()<<std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    delete volume;
}

std::string PointObjectGenerator::getErrorMessage( const size_t maxMemory ) const
{
    std::string errorMessage( "" );
    const size_t totalMemory = sizeof( float ) * object->nvertices() + sizeof( float ) * object->nnormals() + sizeof( char ) * object->ncolors();
    if( totalMemory > maxMemory )
    {
        char ms[512];
        sprintf( ms, "Memory Error: TotalMemory = %zd, MaxMemory = %zd\n", totalMemory, maxMemory );
        errorMessage += ms;
    }
    return errorMessage;
}

kvs::PointObject* PointObjectGenerator::sampling( Argument& param, Camera& camera, kvs::VolumeObjectBase* volume, size_t subpixel_level, float sampling_step )
{
    kvs::TransferFunction& tf = param.transfer_function;

//    tf.setRange(volume);
//    param.transfunc_synthesizer->transfunc_map()["t1"] = tf;
//    param.transfunc_synthesizer->create();

    // volume calculate test.
    if ( param.test_volume )
    {
        return new kvs::TestVolume( &camera, volume, subpixel_level, sampling_step, tf );
    }

#ifdef CPU_SAMPLING_TIME
    std::cout << std::endl << "CPU - ";
#else
    std::cout << std::endl << "GPU - ";
#endif
    switch ( param.sampling_method )
    {
    case 'u':
          std::cout << "Uniform sampling" << std::endl;
          return new kvs::CellByCellUniformSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                     param.transfunc_synthesizer, param.normal_ingred, param.crap,
                                                     param.particle_density, param.batch );
    case 'r':
        std::cout << "Rejection sampling" << std::endl;
        return new kvs::CellByCellRejectionSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                      param.transfunc_synthesizer, param.crap,
                                                      param.particle_density, param.batch );
    case 'm':
        std::cout << "Metropolis sampling" << std::endl;
        return new kvs::CellByCellMetropolisSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                      param.transfunc_synthesizer, param.normal_ingred, param.crap, 
                                                      param.particle_density, param.batch );
    default:
        std::cout << "Unknown sampling method:" << param.sampling_method << std::endl;
        exit( 1 );
        return 0;
    }
}

kvs::UnstructuredVolumeObject* PointObjectGenerator::read_avsucd( char* filename, const int componentId )
{
    kvs::AVSUcd* ucd = new kvs::AVSUcd( filename, 0, componentId );

    kvs::UnstructuredVolumeObject* volume = new kvs::UnstructuredVolumeObject();
    volume->setVeclen( ucd->veclens()[ ucd->componentID() ] );
    volume->setNNodes( ucd->nnodes() );
    volume->setNCells( ucd->nelements() );
    //消費するビデオメモリを抑えるため.
    volume->setCellType( kvs::UnstructuredVolumeObject::Tetrahedra );
    volume->setCoords( ucd->coords() );
    volume->setConnections( ucd->connections() );
    volume->setValues( kvs::AnyValueArray( ucd->values() ) );
    volume->updateMinMaxCoords();
    volume->updateMinMaxValues();

    delete ucd;
    return volume;
}
