#include "ParticleTransferClient.h"
#include "Camera.h"
#include "TransferFunction.h"
#include "PointObject.h"
#include "KVSMLObjectPointWriter.h"
#include "Vector3.h"

#include <iostream>
#include <sstream>
#include <cstring>
#include <fstream>
#include <string>

#ifdef KMATH
#include <kmath_random.h>
#endif

#include "Argument.h"
#include "ParamInfo.h"

#ifdef KMATH
KMATH_Random km_random;
#endif

int main( int argc, char* argv[] )
{
    jpv::ParticleTransferClient client( "localhost",60000 );
    jpv::ParticleTransferClientMessage clntMes;
    jpv::ParticleTransferServerMessage servMes;

    kvs::Camera camera;
    kvs::TransferFunction transfunc;

    Argument param( argc, argv );

    client.initClient();

    // Send inputDir
    strncpy( clntMes.header,"JPTP /1.0\r\n",11 );
    clntMes.initParam = -3;
    clntMes.inputDir = "bearing/bearing";
    clntMes.messageSize = clntMes.byteSize();

    client.sendMessage( clntMes );

    // Recive server condition
    client.recvMessage( servMes );
    int numvol = servMes.numVolDiv;
    int numstep = servMes.numStep;
    kvs::Vector3f minObjectCoord( servMes.minObjectCoord );
    kvs::Vector3f maxObjectCoord( servMes.maxObjectCoord );
    float minValue = servMes.minValue;
    float maxValue = servMes.maxValue;
    std::cout << "minValue: " << minValue << " maxValue: " << maxValue << std::endl;
    std::cout << "minObjectCoord: " << minObjectCoord << std::endl;
    std::cout << "maxObjectCoord: " << maxObjectCoord << std::endl;

    transfunc = param.transfer_function;
    // Loop for time steps.
    for ( int s = 1; s <= numstep; s++ )
    {

        strncpy( clntMes.header,"JPTP /1.0\r\n",11 );
        clntMes.initParam = 1;
        clntMes.samplingMethod = 'u';
        clntMes.repeatLevel = 4;
        clntMes.subPixelLevel = 1;
        clntMes.samplingStep = 3.0;
//    clntMes.repeatLevel = 1;
//    clntMes.subPixelLevel = 2;
//    clntMes.samplingStep = 1;
        clntMes.camera = &camera;
        clntMes.transParam = 0;
        clntMes.timeParam = 2;
        clntMes.step      = s;
        clntMes.nodeType  = 'a';
        clntMes.shuffleMethod = 'r';

#if 0
        if( param.param_file != "" )
        {
            ParamInfo pa;
            if( pa.LoadIN( param.param_file ) )
            {
                clntMes.transferFunctionSynthesis  = pa.getString( "tf_synthesis" );

                clntMes.transfunc.clear();
                clntMes.voleqn.clear();
                for( size_t i = 0; i < 5; i++ )
                {
                    std::stringstream ss;
                    ss << "t" << ( i + 1 );
                    const std::string name  = ss.str();
                    const std::string vname = name + "_var";

                    if( pa.getState( name ) )
                    {
                        NamedTransferFunction tf;
                        jpv::ParticleTransferClientMessage::VolumeEquation ve;

                        tf.Name = name;
                        tf.Resolution = static_cast<size_t>( pa.getInt( "tf_resolution" ) );
                        tf.EquationR = pa.getString( name + "_r" );
                        tf.EquationG = pa.getString( name + "_g" );
                        tf.EquationB = pa.getString( name + "_b" );
                        tf.EquationA = pa.getString( name + "_a" );
                        tf.VarMin = pa.getFloat( name + "_min" );
                        tf.VarMax = pa.getFloat( name + "_max" );

                        const std::string var = pa.getString( vname );
                        tf.Var  = vname;
                        ve.Name = vname;
                        ve.Equation = var;

                        clntMes.transfunc.push_back(tf);
                        clntMes.voleqn.push_back(ve);
                    }
                }
            }
        }
        else
        {
            clntMes.transfunc.clear();
            clntMes.voleqn.clear();

            clntMes.transfunc.push_back( NamedTransferFunction() );
            clntMes.voleqn.push_back( jpv::ParticleTransferClientMessage::VolumeEquation() );

            clntMes.transfunc[0].Name = "t1";
            clntMes.transfunc[0].Var  = "t1_var";
            clntMes.transfunc[0].VarMin = servMes.minValue;
            clntMes.transfunc[0].VarMax = servMes.maxValue;
            clntMes.transfunc[0].selection = NamedTransferFunction::SelectTransferFunction;

            clntMes.voleqn[0].Name = "t1_var";
            clntMes.voleqn[0].Equation = "q1";

            clntMes.transferFunctionSynthesis  = "t1";
        }
#else
        clntMes.transfunc = std::vector<NamedTransferFunction>(3);
        clntMes.voleqn = std::vector<jpv::ParticleTransferClientMessage::VolumeEquation>(3*2);

        clntMes.transfunc[0].Name       = "t1";
        clntMes.transfunc[0].Resolution = 256;
        clntMes.transfunc[0].ColorVar    = "_t1_var_c";
        clntMes.transfunc[0].ColorVarMin = 0;
        clntMes.transfunc[0].ColorVarMax = 50;
        clntMes.transfunc[0].OpacityVar    = "_t1_var_o";
        clntMes.transfunc[0].OpacityVarMin = 0;
        clntMes.transfunc[0].OpacityVarMax = 1;
        clntMes.transfunc[0].EquationR  = "5*x";
        clntMes.transfunc[0].EquationG  = "0";
        clntMes.transfunc[0].EquationB  = "0";
        clntMes.transfunc[0].EquationA  = "0.2 * x";

        clntMes.transfunc[1].Name       = "t2";
        clntMes.transfunc[1].Resolution = 256;
        clntMes.transfunc[1].ColorVar    = "_t2_var_c";
        clntMes.transfunc[1].ColorVarMin = 0;
        clntMes.transfunc[1].ColorVarMax = 25;
        clntMes.transfunc[1].OpacityVar    = "_t2_var_o";
        clntMes.transfunc[1].OpacityVarMin = 0;
        clntMes.transfunc[1].OpacityVarMax = 25;
        clntMes.transfunc[1].EquationR  = "0";
        clntMes.transfunc[1].EquationG  = "10*x";
        clntMes.transfunc[1].EquationB  = "0";
        clntMes.transfunc[1].EquationA  = "0.1";

        clntMes.transfunc[2].Name       = "t3";
        clntMes.transfunc[2].Resolution = 256;
        clntMes.transfunc[2].ColorVar    = "_t3_var_c";
        clntMes.transfunc[2].ColorVarMin = -60;
        clntMes.transfunc[2].ColorVarMax = 10;
        clntMes.transfunc[2].OpacityVar    = "_t3_var_o";
        clntMes.transfunc[2].OpacityVarMin = -60;
        clntMes.transfunc[2].OpacityVarMax = 10;
        clntMes.transfunc[2].EquationR  = "0";
        clntMes.transfunc[2].EquationG  = "0";
        clntMes.transfunc[2].EquationB  = "3*(x+60)";
        clntMes.transfunc[2].EquationA  = "0";

        clntMes.voleqn[0].Name = "_t1_var_c";
        clntMes.voleqn[0].Equation = "X";
        clntMes.voleqn[1].Name = "_t2_var_c";
        clntMes.voleqn[1].Equation = "Y";
        clntMes.voleqn[2].Name = "_t3_var_c";
        clntMes.voleqn[2].Equation = "Z";
        clntMes.voleqn[3].Name = "_t1_var_o";
        clntMes.voleqn[3].Equation = "q1";
        clntMes.voleqn[4].Name = "_t2_var_o";
        clntMes.voleqn[4].Equation = "q2";
        clntMes.voleqn[5].Name = "_t3_var_o";
        clntMes.voleqn[5].Equation = "q3";

        clntMes.transferFunctionSynthesis = "t1+t2+t3";
#endif
        clntMes.enable_crap_region = false;
        clntMes.crap_region[0] = 0.0;
        clntMes.crap_region[1] = 0.0;
        clntMes.crap_region[2] = 0.0;
        clntMes.crap_region[3] = 10.0;
        clntMes.crap_region[4] = 4.0;
        clntMes.crap_region[5] = 10.0;

        clntMes.messageSize = clntMes.byteSize();
        client.sendMessage( clntMes );

        kvs::PointObject object;

        // Loop for sub volumes.
        for ( int n = 0; n < numvol; n++ )
        {

            client.recvMessage( servMes );

            int32_t nmemb = servMes.numParticle * 3;
            if ( nmemb == 0 ) continue;
            kvs::ValueArray<kvs::Real32> coords_array( servMes.positions, nmemb );
            kvs::ValueArray<kvs::UInt8>  colors_array( servMes.colors, nmemb );
            kvs::ValueArray<kvs::Real32> normals_array( servMes.normals, nmemb );
            kvs::PointObject obj;
            obj.setCoords( coords_array );
            obj.setColors( colors_array );
            obj.setNormals( normals_array );

            object += obj;

            std::cout << "t1_var_c " << servMes.varRange.min("t1_var_c") << " " << servMes.varRange.max("t1_var_c") << std::endl;
            std::cout << "t1_var_o " << servMes.varRange.min("t1_var_o") << " " << servMes.varRange.max("t1_var_o") << std::endl;
            std::cout << "t2_var_c " << servMes.varRange.min("t2_var_c") << " " << servMes.varRange.max("t2_var_c") << std::endl;
            std::cout << "t2_var_o " << servMes.varRange.min("t2_var_o") << " " << servMes.varRange.max("t2_var_o") << std::endl;
            std::cout << "t3_var_c " << servMes.varRange.min("t3_var_c") << " " << servMes.varRange.max("t3_var_c") << std::endl;
            std::cout << "t3_var_o " << servMes.varRange.min("t3_var_o") << " " << servMes.varRange.max("t3_var_o") << std::endl;
            std::cout << "t4_var_c " << servMes.varRange.min("t4_var_c") << " " << servMes.varRange.max("t4_var_c") << std::endl;
            std::cout << "t4_var_o " << servMes.varRange.min("t4_var_o") << " " << servMes.varRange.max("t4_var_o") << std::endl;
            std::cout << "t5_var_c " << servMes.varRange.min("t5_var_c") << " " << servMes.varRange.max("t5_var_c") << std::endl;
            std::cout << "t5_var_o " << servMes.varRange.min("t5_var_o") << " " << servMes.varRange.max("t5_var_o") << std::endl;

            obj.clear();
            delete[] servMes.positions;
            delete[] servMes.colors;
            delete[] servMes.normals;
        }

        std::stringstream suffix;
        suffix << '.' << std::setw( 4 ) << std::setfill( '0' ) << s;
        std::string basename = "client-out" + suffix.str();
        KVSMLObjectPointWriter( &object, basename );

        object.clear();
    }

    strncpy( clntMes.header,"JPTP /1.0\r\n",11 );
    clntMes.initParam = -2;
    clntMes.messageSize = clntMes.byteSize();
    client.sendMessage( clntMes );
    client.recvMessage( servMes );
    client.termClient();

    return 0;
}

