#ifndef __TRANSFER_FUNCTION_CONFIGURATOR_H__
#define __TRANSFER_FUNCTION_CONFIGURATOR_H__

#include <map>
#include <string>
#include <sstream>
#include <vector>

#include "TransferFunction.h"
#include "TransferFunctionSynthesizer.h"
#include "ParticleTransferProtocol.h"
#include "ExtendedTransferFunction.h"
#include "ParamInfo.h"
#include "FilterInfo.h"

const static size_t TF_COUNT = 5;

class TransferFunctionSynthesizerCreator
{
public:
    struct VolumeEquation {
        std::string Name;
        std::string Equation;
    };

public:
    TransferFunctionSynthesizerCreator();
    ~TransferFunctionSynthesizerCreator();
    TransferFunctionSynthesizer* create();

    void setFilterInfo( const FilterInfo& fi );
    void initQuantityMap( const int32_t numIngredients );
    void setRange( const std::string name, const float min_value, const float max_value );
    void setColorRange( const std::string& name, const float min_value, const float max_value );
    void setOpacityRange( const std::string& name, const float min_value, const float max_value );
    void setProtocol(  const jpv::ParticleTransferClientMessage& clntMes );
    void setParamInfo(  const ParamInfo& pa );
    void setAsisTransferFunction( const kvs::TransferFunction tf );
    void setTransferFunction( const std::string& name, const kvs::TransferFunction& tf );

private:
    TransferFunctionSynthesizer* m_synthesizer;
    std::string m_transfunc_synthesis;

    std::vector<NamedTransferFunction> m_transfunc;
    std::vector<VolumeEquation>        m_voleqn;

private:
    void assign();
    void set_protocol(  const jpv::ParticleTransferClientMessage& clntMes );
    void set_param_info(  const ParamInfo& pa );

    QuantityMap m_qmap;
};


#endif //__TRANSFER_FUNCTION_CONFIGURATOR_H__
