#include "../FunctionParser/function.h"
#include "../FunctionParser/function_op.h"
#include "../FunctionParser/function_parser.h"

#include "../TransferFunction.h"

#include <kmath_random.h>
KMATH_Random km_random;

#include <map>
typedef std::map<const std::string, const kvs::TransferFunction*>
        TransferFunctionMap;

const kvs::TransferFunction* InitFromFunction(std::string rfstr,
                                              std::string gfstr,
                                              std::string bfstr,
                                              std::string ofstr,
                                              const size_t resolution,
                                              const float  min_value,
                                              const float  max_value)
{
  kvs::ColorMap   *c = new kvs::ColorMap(resolution,min_value,max_value);
  kvs::OpacityMap *a = new kvs::OpacityMap(resolution,min_value,max_value);

  FuncParser::Variables vars;
  FuncParser::Variable var_x;
  FuncParser::Function rf,gf,bf,of;

  var_x.tag("x");
  vars.push_back( var_x );

  FuncParser::FunctionParser rf_parse( rfstr, rfstr.size() + 1 );
  FuncParser::FunctionParser gf_parse( gfstr, gfstr.size() + 1 );
  FuncParser::FunctionParser bf_parse( bfstr, bfstr.size() + 1 );
  FuncParser::FunctionParser of_parse( ofstr, ofstr.size() + 1 );
  FuncParser::FunctionParser::Error err_r = rf_parse.express( rf, vars );
  FuncParser::FunctionParser::Error err_g = gf_parse.express( gf, vars );
  FuncParser::FunctionParser::Error err_b = bf_parse.express( bf, vars );
  FuncParser::FunctionParser::Error err_o = of_parse.express( of, vars );

  const float stride = (max_value - min_value) / (resolution - 1);
  float x = min_value;
  for (size_t i = 0; i < resolution; ++i, x+= stride) {
    kvs::UInt8 r,g,b;
    float opacity;

    var_x = x;
    r = rf.eval();
    g = gf.eval();
    b = bf.eval();
    opacity = of.eval();

    kvs::RGBColor color(r,g,b);
    c->addPoint(x, color);
    a->addPoint(x, opacity);
  }
  c->create();
  a->create();

  kvs::TransferFunction *t = new kvs::TransferFunction(*c,*a);
  delete c,a;

  return t;
}

int main(int argc, char* argv[])
{

  const float  min_value  =   0.0;
  const float  max_value  = 100.0;
  const size_t resolution = 100;

  // right side eqn of transfer function "y=f(x)"
  const std::string func_red   = "x";
  const std::string func_green = "256*x/100";
  const std::string func_blue  = "x+x";
  const std::string func_opcty = "sin(x)";

  TransferFunctionMap primitive_transfer_function_map;

  // init transfer function
  const kvs::TransferFunction *t1;
  if (true) {
    // Init TransferFunction instance from function string
    t1 = InitFromFunction(func_red,func_green,func_blue,func_opcty,
                          resolution,min_value,max_value);
  } else {
    // default transfer function
    kvs::TransferFunction *t0 = new kvs::TransferFunction(resolution);
    t0->setRange(min_value,max_value);
    t1 = t0;
  }

  primitive_transfer_function_map["t1"] = t1;


  // print transfer function
  const kvs::TransferFunction *t = primitive_transfer_function_map["t1"];
  const kvs::ColorMap&   c = t->colorMap();
  const kvs::OpacityMap& a = t->opacityMap();
  std::cout << "hasRange      : " << (t->hasRange()?'1':'0') << std::endl;
  std::cout << "min,max Value : " << t->minValue() << ',' << t->maxValue() << std::endl;
  std::cout << "resolution    : " << t->resolution() << std::endl;
  for (size_t i = 0; i < resolution; ++i) {
    const kvs::Real32 opcty = a[i];
    const int r = c[i].r();
    const int g = c[i].g();
    const int b = c[i].b();
    std::cout << i << " " << r << "," << g << "," << b << "," << opcty << std::endl;
  }

  delete primitive_transfer_function_map["t1"];

  return 0;
}


