/****************************************************************************/
/**
 *  @file CellByCellMetropolisSampling.h
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: CellByCellMetropolisSampling.h 634 2010-10-13 07:04:05Z naohisa.sakamoto $
 */
/****************************************************************************/
#ifndef KVS__TEST__CELL_BY_CELL_METROPOLIS_SAMPLING_H_INCLUDE
#define KVS__TEST__CELL_BY_CELL_METROPOLIS_SAMPLING_H_INCLUDE

#include "MapperBase.h"
#include "Camera.h"
#include "PointObject.h"
#include "VolumeObjectBase.h"
#include "StructuredVolumeObject.h"
#include "UnstructuredVolumeObject.h"
#include "ClassName.h"
#include "Module.h"
#include "CellByCellParticleGenerator.h"
#include "CrapRegion.h"
#include "TransferFunctionSynthesizer.h"
#include "TransferFunctionSynthesizerCreator.h"


namespace kvs
{
//namespace test
//{

/*===========================================================================*/
/**
 *  @brief  Cell-by-cell particle generation class.
 */
/*===========================================================================*/
class CellByCellMetropolisSampling : public kvs::MapperBase, public kvs::PointObject
{
    // Class name.
    kvsClassName( kvs::CellByCellMetropolisSampling );

    // Module information.
    kvsModuleCategory( Mapper );
    kvsModuleBaseClass( kvs::MapperBase );
    kvsModuleSuperClass( kvs::PointObject );

private:

    const kvs::Camera*     m_camera;         ///< camera (reference)
    size_t                 m_subpixel_level; ///< subpixel level
    float                  m_sampling_step;  ///< sampling step in the object coordinate
    float                  m_object_depth;   ///< object depth
    kvs::ValueArray<float> m_density_map;    ///< density map
    CrapRegion             m_crap;
    TransferFunctionSynthesizer* m_transfunc_synthesizer;

    const size_t m_normal_ingred;

    float                  m_particle_density;
    bool                   m_batch;

public:

    CellByCellMetropolisSampling( void );

    CellByCellMetropolisSampling(
        const kvs::VolumeObjectBase* volume,
        const size_t                 subpixel_level,
        const float                  sampling_step,
        const kvs::TransferFunction& transfer_function,
        TransferFunctionSynthesizer* transfunc_synthesizer,
        const size_t                 normal_ingred,
        const CrapRegion&            crap,
        const float                  object_depth = 0.0f );

    CellByCellMetropolisSampling(
        const kvs::Camera*           camera,
        const kvs::VolumeObjectBase* volume,
        const size_t                 subpixel_level,
        const float                  sampling_step,
        const kvs::TransferFunction& transfer_function,
        TransferFunctionSynthesizer* transfunc_synthesizer,
        const size_t                 normal_ingred,
        const CrapRegion&            crap,
        const float                  particle_density,
        const bool                   batch,
        const float                  object_depth = 0.0f );

    virtual ~CellByCellMetropolisSampling( void );

public:

    SuperClass* exec( const kvs::ObjectBase* object );

public:

    const size_t subpixelLevel( void ) const;

    const float samplingStep( void ) const;

    const float objectDepth( void ) const;

    void attachCamera( const kvs::Camera* camera );

    void setSubpixelLevel( const size_t subpixel_level );

    void setSamplingStep( const float sampling_step );

    void setObjectDepth( const float object_depth );

private:

    void mapping( const kvs::Camera* camera, const kvs::StructuredVolumeObject* volume );

    void mapping( const kvs::Camera* camera, const kvs::UnstructuredVolumeObject* volume );

    template <typename T>
    void generate_particles( const kvs::StructuredVolumeObject* volume );

    template <typename T>
    void generate_particles( const kvs::UnstructuredVolumeObject* volume );

    /*** added by MHIR on /12/12/12/ ***/
    /*
    const float calculate_density( const float scalar );
    const float calculate_volume ();
    const size_t calculate_number_of_particles(
    				       const float density,
    				       const float volume_of_cell
    				       );
    */
    /***********************************/
};

//} // end of namespace test
} // end of namespace kvs

#endif // KVS__CELL_BY_CELL_METROPOLIS_SAMPLING_H_INCLUDE
