#ifndef DISTRIBUTED_UNIFORM_SAMPLING_H
#define DISTRIBUTED_UNIFORM_SAMPLING_H

#include "UnstructuredVolumeObject.h"

namespace kvs
{
//namespace test {

class DistributedUniformSampling
{
public:

    static void generate_particles_gt5d_master_gpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* density_map,
        const unsigned char* color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>& vertex_coords,
        kvs::ValueArray<kvs::UInt8>&  vertex_colors,
        kvs::ValueArray<kvs::Real32>& vertex_normals );

    static void generate_particles_gt5d_slave_gpu(
        const kvs::UnstructuredVolumeObject* volume );

    static void generate_particles_gt5d_gpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* density_map,
        const unsigned char* color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>* vertex_coords,
        kvs::ValueArray<kvs::UInt8>*  vertex_colors,
        kvs::ValueArray<kvs::Real32>* vertex_normals );

private:
    static void generate_particles_core_gpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* density_map,
        const unsigned char* color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>* vertex_coords,
        kvs::ValueArray<kvs::UInt8>*  vertex_colors,
        kvs::ValueArray<kvs::Real32>* vertex_normals );

public:
    static void generate_particles_master_gpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* density_map,
        const unsigned char* color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>& vertex_coords,
        kvs::ValueArray<kvs::UInt8>&  vertex_colors,
        kvs::ValueArray<kvs::Real32>& vertex_normals );

    static void generate_particles_slave_gpu();

    /*** debug ***/
    static void generate_particles_slave_ts_gpu(
        const kvs::UnstructuredVolumeObject* volume );
    /*************/
private:
    static void generate_particles_core_cpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* raw_density_map,
        const unsigned char* raw_color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>* vertex_coords,
        kvs::ValueArray<kvs::UInt8>*  vertex_colors,
        kvs::ValueArray<kvs::Real32>* vertex_normals );

public:
    static void generate_particles_master_cpu(
        const kvs::UnstructuredVolumeObject* volume,
        const float* density_map,
        const unsigned char* color_map,
        size_t resolution, float min_value, float max_value,
        kvs::ValueArray<kvs::Real32>& vertex_coords,
        kvs::ValueArray<kvs::UInt8>&  vertex_colors,
        kvs::ValueArray<kvs::Real32>& vertex_normals );

    static void generate_particles_slave_cpu();

};

//} // end of namespace test
} // end of namespace kvs

#endif

