#ifndef __POINT_OBJECT_GENERATOR_H__
#define __POINT_OBJECT_GENERATOR_H__

#include <vector>
#include <string>
#include "UnstructuredVolumeObject.h"
#include "FilterInfo.h"

class Argument;

namespace kvs
{
class PointObject;
class VolumeObjectBase;
class UnstructuredVolumeObject;
class Camera;

class PointObjectGenerator
{
private:
    kvs::PointObject* object;
    const FilterInfo*   m_fi;

public:

    PointObjectGenerator() :object( 0 ) {};

    void createTestObject( const size_t howMany );

    void createHexahedraPointObject(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step );

    void createPyramidPointObject(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step );

    void createQuadraticTetrahedra(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step );

    void createQuadraticHexahedra(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step );

    void createHTTR(
        Argument& param, Camera& camera, size_t subpixel_level,float sampling_step,size_t step );

    void createGT5D(
        Argument& param, Camera& camera, size_t subpixel_level,
        float sampling_step, kvs::UnstructuredVolumeObject* volume );

    void createGT5D_full(
        Argument& param, Camera& camera, size_t subpixel_level,
        float sampling_step, kvs::UnstructuredVolumeObject* volume );

    void createFromFile(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step );

    void createFromFile(
        Argument& param, Camera& camera, size_t subpixel_level, float sampling_step, int st, int vl);

    kvs::PointObject* getPointObject()
    {
        return object;
    }

    std::string getErrorMessage( const size_t maxMemory ) const;

    void setFinlterInfo( const FilterInfo& fi )
    {
        m_fi = &fi;
    }

private:
    kvs::PointObject* sampling( Argument& param, Camera& camera, kvs::VolumeObjectBase* volume, size_t subpixel_level, float sampling_step );

    kvs::UnstructuredVolumeObject* read_avsucd( char* filename, const int componentId );
};

}

#endif
