#ifndef __KVS_VISCLIENT_PROFILER_H__
#define __KVS_VISCLIENT_PROFILER_H__

#include <ctime>

#include <vector>
#include <map>
#include <string>

#include "common.h"

#include <iostream>
#include <fstream>

namespace kvs
{
namespace visclient
{

class Profiler
{
private:
    Profiler( void ) {};

    ~Profiler( void ) {};

public:
    static Profiler* get()
    {
        static Profiler profiler;
        return &profiler;
    }

    void init()
    {
        strAndValues.clear();
        strAndTimes.clear();
    }

    void start( const std::string& profileName )
    {
        strAndTimes[profileName] = get_time();
    }

    void end( const std::string& profileName )
    {
        strAndTimes[profileName] = get_time() - strAndTimes[profileName];
    }

    void set( const std::string& profileName, const int value )
    {
        strAndValues[profileName] = value;
    }

    void writeToTerminal() const
    {
        for( std::map<std::string, int>::const_iterator iter = strAndValues.begin(); iter != strAndValues.end(); ++iter )
        {
            std::cout << iter->first << " : " << iter->second << std::endl;
        }
        std::cout.precision( 4 );
        for( std::map<std::string, double>::const_iterator iter = strAndTimes.begin(); iter != strAndTimes.end(); ++iter )
        {
            std::cout << iter->first << " : " << iter->second << "[ms]" << std::endl;
        }
    }

    void writeToCSV( const int timeStep, const std::string& fileName ) const
    {
        std::ofstream stream( fileName.c_str(), std::ios::app );

        if( timeStep == 1 )
        {
            stream << "time step" << ",";
            for( std::map<std::string, int>::const_iterator iter = strAndValues.begin(); iter != strAndValues.end(); ++iter )
            {
                stream << iter->first << ",";
            }
            for( std::map<std::string, double>::const_iterator iter = strAndTimes.begin(); iter != strAndTimes.end(); ++iter )
            {
                stream << iter->first << "[ms]" << ",";
            }
            stream << std::endl;
        }

        stream << timeStep << ",";
        for( std::map<std::string, int>::const_iterator iter = strAndValues.begin(); iter != strAndValues.end(); ++iter )
        {
            stream << iter->second << ",";
        }
        for( std::map<std::string, double>::const_iterator iter = strAndTimes.begin(); iter != strAndTimes.end(); ++iter )
        {
            stream << iter->second << ",";
        }
        stream << std::endl;
    }

private:
    std::map< std::string, int > strAndValues;
    std::map< std::string, double > strAndTimes;
};

}
}

#endif
